# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: tendermint/abci/types.proto
# plugin: python-betterproto
# This file has been @generated
from dataclasses import dataclass
from datetime import datetime
from typing import (
    TYPE_CHECKING,
    Dict,
    List,
    Optional,
)

import betterproto
import grpclib
from betterproto.grpc.grpclib_server import ServiceBase

from .. import (
    crypto as _crypto__,
    types as _types__,
)


if TYPE_CHECKING:
    import grpclib.server
    from betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


class CheckTxType(betterproto.Enum):
    NEW = 0
    RECHECK = 1


class EvidenceType(betterproto.Enum):
    UNKNOWN = 0
    DUPLICATE_VOTE = 1
    LIGHT_CLIENT_ATTACK = 2


class ResponseOfferSnapshotResult(betterproto.Enum):
    UNKNOWN = 0
    ACCEPT = 1
    ABORT = 2
    REJECT = 3
    REJECT_FORMAT = 4
    REJECT_SENDER = 5


class ResponseApplySnapshotChunkResult(betterproto.Enum):
    UNKNOWN = 0
    ACCEPT = 1
    ABORT = 2
    RETRY = 3
    RETRY_SNAPSHOT = 4
    REJECT_SNAPSHOT = 5


@dataclass(eq=False, repr=False)
class Request(betterproto.Message):
    echo: 'RequestEcho' = betterproto.message_field(1, group='value')
    flush: 'RequestFlush' = betterproto.message_field(2, group='value')
    info: 'RequestInfo' = betterproto.message_field(3, group='value')
    set_option: 'RequestSetOption' = betterproto.message_field(4, group='value')
    init_chain: 'RequestInitChain' = betterproto.message_field(5, group='value')
    query: 'RequestQuery' = betterproto.message_field(6, group='value')
    begin_block: 'RequestBeginBlock' = betterproto.message_field(7, group='value')
    check_tx: 'RequestCheckTx' = betterproto.message_field(8, group='value')
    deliver_tx: 'RequestDeliverTx' = betterproto.message_field(9, group='value')
    end_block: 'RequestEndBlock' = betterproto.message_field(10, group='value')
    commit: 'RequestCommit' = betterproto.message_field(11, group='value')
    list_snapshots: 'RequestListSnapshots' = betterproto.message_field(
        12, group='value'
    )
    offer_snapshot: 'RequestOfferSnapshot' = betterproto.message_field(
        13, group='value'
    )
    load_snapshot_chunk: 'RequestLoadSnapshotChunk' = betterproto.message_field(
        14, group='value'
    )
    apply_snapshot_chunk: 'RequestApplySnapshotChunk' = betterproto.message_field(
        15, group='value'
    )


@dataclass(eq=False, repr=False)
class RequestEcho(betterproto.Message):
    message: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class RequestFlush(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class RequestInfo(betterproto.Message):
    version: str = betterproto.string_field(1)
    block_version: int = betterproto.uint64_field(2)
    p2_p_version: int = betterproto.uint64_field(3)


@dataclass(eq=False, repr=False)
class RequestSetOption(betterproto.Message):
    """nondeterministic"""

    key: str = betterproto.string_field(1)
    value: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class RequestInitChain(betterproto.Message):
    time: datetime = betterproto.message_field(1)
    chain_id: str = betterproto.string_field(2)
    consensus_params: 'ConsensusParams' = betterproto.message_field(3)
    validators: List['ValidatorUpdate'] = betterproto.message_field(4)
    app_state_bytes: bytes = betterproto.bytes_field(5)
    initial_height: int = betterproto.int64_field(6)


@dataclass(eq=False, repr=False)
class RequestQuery(betterproto.Message):
    data: bytes = betterproto.bytes_field(1)
    path: str = betterproto.string_field(2)
    height: int = betterproto.int64_field(3)
    prove: bool = betterproto.bool_field(4)


@dataclass(eq=False, repr=False)
class RequestBeginBlock(betterproto.Message):
    hash: bytes = betterproto.bytes_field(1)
    header: '_types__.Header' = betterproto.message_field(2)
    last_commit_info: 'LastCommitInfo' = betterproto.message_field(3)
    byzantine_validators: List['Evidence'] = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class RequestCheckTx(betterproto.Message):
    tx: bytes = betterproto.bytes_field(1)
    type: 'CheckTxType' = betterproto.enum_field(2)


@dataclass(eq=False, repr=False)
class RequestDeliverTx(betterproto.Message):
    tx: bytes = betterproto.bytes_field(1)


@dataclass(eq=False, repr=False)
class RequestEndBlock(betterproto.Message):
    height: int = betterproto.int64_field(1)


@dataclass(eq=False, repr=False)
class RequestCommit(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class RequestListSnapshots(betterproto.Message):
    """lists available snapshots"""

    pass


@dataclass(eq=False, repr=False)
class RequestOfferSnapshot(betterproto.Message):
    """offers a snapshot to the application"""

    snapshot: 'Snapshot' = betterproto.message_field(1)
    app_hash: bytes = betterproto.bytes_field(2)


@dataclass(eq=False, repr=False)
class RequestLoadSnapshotChunk(betterproto.Message):
    """loads a snapshot chunk"""

    height: int = betterproto.uint64_field(1)
    format: int = betterproto.uint32_field(2)
    chunk: int = betterproto.uint32_field(3)


@dataclass(eq=False, repr=False)
class RequestApplySnapshotChunk(betterproto.Message):
    """Applies a snapshot chunk"""

    index: int = betterproto.uint32_field(1)
    chunk: bytes = betterproto.bytes_field(2)
    sender: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class Response(betterproto.Message):
    exception: 'ResponseException' = betterproto.message_field(1, group='value')
    echo: 'ResponseEcho' = betterproto.message_field(2, group='value')
    flush: 'ResponseFlush' = betterproto.message_field(3, group='value')
    info: 'ResponseInfo' = betterproto.message_field(4, group='value')
    set_option: 'ResponseSetOption' = betterproto.message_field(5, group='value')
    init_chain: 'ResponseInitChain' = betterproto.message_field(6, group='value')
    query: 'ResponseQuery' = betterproto.message_field(7, group='value')
    begin_block: 'ResponseBeginBlock' = betterproto.message_field(8, group='value')
    check_tx: 'ResponseCheckTx' = betterproto.message_field(9, group='value')
    deliver_tx: 'ResponseDeliverTx' = betterproto.message_field(10, group='value')
    end_block: 'ResponseEndBlock' = betterproto.message_field(11, group='value')
    commit: 'ResponseCommit' = betterproto.message_field(12, group='value')
    list_snapshots: 'ResponseListSnapshots' = betterproto.message_field(
        13, group='value'
    )
    offer_snapshot: 'ResponseOfferSnapshot' = betterproto.message_field(
        14, group='value'
    )
    load_snapshot_chunk: 'ResponseLoadSnapshotChunk' = betterproto.message_field(
        15, group='value'
    )
    apply_snapshot_chunk: 'ResponseApplySnapshotChunk' = betterproto.message_field(
        16, group='value'
    )


@dataclass(eq=False, repr=False)
class ResponseException(betterproto.Message):
    """nondeterministic"""

    error: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class ResponseEcho(betterproto.Message):
    message: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class ResponseFlush(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class ResponseInfo(betterproto.Message):
    data: str = betterproto.string_field(1)
    version: str = betterproto.string_field(2)
    app_version: int = betterproto.uint64_field(3)
    last_block_height: int = betterproto.int64_field(4)
    last_block_app_hash: bytes = betterproto.bytes_field(5)


@dataclass(eq=False, repr=False)
class ResponseSetOption(betterproto.Message):
    """nondeterministic"""

    code: int = betterproto.uint32_field(1)
    log: str = betterproto.string_field(3)
    """bytes data = 2;"""

    info: str = betterproto.string_field(4)


@dataclass(eq=False, repr=False)
class ResponseInitChain(betterproto.Message):
    consensus_params: 'ConsensusParams' = betterproto.message_field(1)
    validators: List['ValidatorUpdate'] = betterproto.message_field(2)
    app_hash: bytes = betterproto.bytes_field(3)


@dataclass(eq=False, repr=False)
class ResponseQuery(betterproto.Message):
    code: int = betterproto.uint32_field(1)
    log: str = betterproto.string_field(3)
    """bytes data = 2; // use "value" instead."""

    info: str = betterproto.string_field(4)
    index: int = betterproto.int64_field(5)
    key: bytes = betterproto.bytes_field(6)
    value: bytes = betterproto.bytes_field(7)
    proof_ops: '_crypto__.ProofOps' = betterproto.message_field(8)
    height: int = betterproto.int64_field(9)
    codespace: str = betterproto.string_field(10)


@dataclass(eq=False, repr=False)
class ResponseBeginBlock(betterproto.Message):
    events: List['Event'] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class ResponseCheckTx(betterproto.Message):
    code: int = betterproto.uint32_field(1)
    data: bytes = betterproto.bytes_field(2)
    log: str = betterproto.string_field(3)
    info: str = betterproto.string_field(4)
    gas_wanted: int = betterproto.int64_field(5)
    gas_used: int = betterproto.int64_field(6)
    events: List['Event'] = betterproto.message_field(7)
    codespace: str = betterproto.string_field(8)


@dataclass(eq=False, repr=False)
class ResponseDeliverTx(betterproto.Message):
    code: int = betterproto.uint32_field(1)
    data: bytes = betterproto.bytes_field(2)
    log: str = betterproto.string_field(3)
    info: str = betterproto.string_field(4)
    gas_wanted: int = betterproto.int64_field(5)
    gas_used: int = betterproto.int64_field(6)
    events: List['Event'] = betterproto.message_field(7)
    codespace: str = betterproto.string_field(8)


@dataclass(eq=False, repr=False)
class ResponseEndBlock(betterproto.Message):
    validator_updates: List['ValidatorUpdate'] = betterproto.message_field(1)
    consensus_param_updates: 'ConsensusParams' = betterproto.message_field(2)
    events: List['Event'] = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class ResponseCommit(betterproto.Message):
    data: bytes = betterproto.bytes_field(2)
    """reserve 1"""

    retain_height: int = betterproto.int64_field(3)


@dataclass(eq=False, repr=False)
class ResponseListSnapshots(betterproto.Message):
    snapshots: List['Snapshot'] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class ResponseOfferSnapshot(betterproto.Message):
    result: 'ResponseOfferSnapshotResult' = betterproto.enum_field(1)


@dataclass(eq=False, repr=False)
class ResponseLoadSnapshotChunk(betterproto.Message):
    chunk: bytes = betterproto.bytes_field(1)


@dataclass(eq=False, repr=False)
class ResponseApplySnapshotChunk(betterproto.Message):
    result: 'ResponseApplySnapshotChunkResult' = betterproto.enum_field(1)
    refetch_chunks: List[int] = betterproto.uint32_field(2)
    reject_senders: List[str] = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class ConsensusParams(betterproto.Message):
    """
    ConsensusParams contains all consensus-relevant parameters
    that can be adjusted by the abci app
    """

    block: 'BlockParams' = betterproto.message_field(1)
    evidence: '_types__.EvidenceParams' = betterproto.message_field(2)
    validator: '_types__.ValidatorParams' = betterproto.message_field(3)
    version: '_types__.VersionParams' = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class BlockParams(betterproto.Message):
    """BlockParams contains limits on the block size."""

    max_bytes: int = betterproto.int64_field(1)
    """Note: must be greater than 0"""

    max_gas: int = betterproto.int64_field(2)
    """Note: must be greater or equal to -1"""


@dataclass(eq=False, repr=False)
class LastCommitInfo(betterproto.Message):
    round: int = betterproto.int32_field(1)
    votes: List['VoteInfo'] = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class Event(betterproto.Message):
    """
    Event allows application developers to attach additional information to
    ResponseBeginBlock, ResponseEndBlock, ResponseCheckTx and ResponseDeliverTx.
    Later, transactions may be queried using these events.
    """

    type: str = betterproto.string_field(1)
    attributes: List['EventAttribute'] = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class EventAttribute(betterproto.Message):
    """EventAttribute is a single key-value pair, associated with an event."""

    key: bytes = betterproto.bytes_field(1)
    value: bytes = betterproto.bytes_field(2)
    index: bool = betterproto.bool_field(3)


@dataclass(eq=False, repr=False)
class TxResult(betterproto.Message):
    """
    TxResult contains results of executing the transaction.
    One usage is indexing transaction results.
    """

    height: int = betterproto.int64_field(1)
    index: int = betterproto.uint32_field(2)
    tx: bytes = betterproto.bytes_field(3)
    result: 'ResponseDeliverTx' = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class Validator(betterproto.Message):
    """Validator"""

    address: bytes = betterproto.bytes_field(1)
    power: int = betterproto.int64_field(3)
    """PubKey pub_key = 2 [(gogoproto.nullable)=false];"""


@dataclass(eq=False, repr=False)
class ValidatorUpdate(betterproto.Message):
    """ValidatorUpdate"""

    pub_key: '_crypto__.PublicKey' = betterproto.message_field(1)
    power: int = betterproto.int64_field(2)


@dataclass(eq=False, repr=False)
class VoteInfo(betterproto.Message):
    """VoteInfo"""

    validator: 'Validator' = betterproto.message_field(1)
    signed_last_block: bool = betterproto.bool_field(2)


@dataclass(eq=False, repr=False)
class Evidence(betterproto.Message):
    type: 'EvidenceType' = betterproto.enum_field(1)
    validator: 'Validator' = betterproto.message_field(2)
    """The offending validator"""

    height: int = betterproto.int64_field(3)
    """The height when the offense occurred"""

    time: datetime = betterproto.message_field(4)
    """The corresponding time where the offense occurred"""

    total_voting_power: int = betterproto.int64_field(5)
    """
    Total voting power of the validator set in case the ABCI application does
    not store historical validators.
    https://github.com/tendermint/tendermint/issues/4581
    """


@dataclass(eq=False, repr=False)
class Snapshot(betterproto.Message):
    height: int = betterproto.uint64_field(1)
    format: int = betterproto.uint32_field(2)
    chunks: int = betterproto.uint32_field(3)
    hash: bytes = betterproto.bytes_field(4)
    metadata: bytes = betterproto.bytes_field(5)


class AbciApplicationStub(betterproto.ServiceStub):
    async def echo(
        self,
        request_echo: 'RequestEcho',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'ResponseEcho':
        return await self._unary_unary(
            '/tendermint.abci.ABCIApplication/Echo',
            request_echo,
            ResponseEcho,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def flush(
        self,
        request_flush: 'RequestFlush',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'ResponseFlush':
        return await self._unary_unary(
            '/tendermint.abci.ABCIApplication/Flush',
            request_flush,
            ResponseFlush,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def info(
        self,
        request_info: 'RequestInfo',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'ResponseInfo':
        return await self._unary_unary(
            '/tendermint.abci.ABCIApplication/Info',
            request_info,
            ResponseInfo,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def set_option(
        self,
        request_set_option: 'RequestSetOption',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'ResponseSetOption':
        return await self._unary_unary(
            '/tendermint.abci.ABCIApplication/SetOption',
            request_set_option,
            ResponseSetOption,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def deliver_tx(
        self,
        request_deliver_tx: 'RequestDeliverTx',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'ResponseDeliverTx':
        return await self._unary_unary(
            '/tendermint.abci.ABCIApplication/DeliverTx',
            request_deliver_tx,
            ResponseDeliverTx,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def check_tx(
        self,
        request_check_tx: 'RequestCheckTx',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'ResponseCheckTx':
        return await self._unary_unary(
            '/tendermint.abci.ABCIApplication/CheckTx',
            request_check_tx,
            ResponseCheckTx,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def query(
        self,
        request_query: 'RequestQuery',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'ResponseQuery':
        return await self._unary_unary(
            '/tendermint.abci.ABCIApplication/Query',
            request_query,
            ResponseQuery,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def commit(
        self,
        request_commit: 'RequestCommit',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'ResponseCommit':
        return await self._unary_unary(
            '/tendermint.abci.ABCIApplication/Commit',
            request_commit,
            ResponseCommit,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def init_chain(
        self,
        request_init_chain: 'RequestInitChain',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'ResponseInitChain':
        return await self._unary_unary(
            '/tendermint.abci.ABCIApplication/InitChain',
            request_init_chain,
            ResponseInitChain,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def begin_block(
        self,
        request_begin_block: 'RequestBeginBlock',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'ResponseBeginBlock':
        return await self._unary_unary(
            '/tendermint.abci.ABCIApplication/BeginBlock',
            request_begin_block,
            ResponseBeginBlock,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def end_block(
        self,
        request_end_block: 'RequestEndBlock',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'ResponseEndBlock':
        return await self._unary_unary(
            '/tendermint.abci.ABCIApplication/EndBlock',
            request_end_block,
            ResponseEndBlock,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def list_snapshots(
        self,
        request_list_snapshots: 'RequestListSnapshots',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'ResponseListSnapshots':
        return await self._unary_unary(
            '/tendermint.abci.ABCIApplication/ListSnapshots',
            request_list_snapshots,
            ResponseListSnapshots,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def offer_snapshot(
        self,
        request_offer_snapshot: 'RequestOfferSnapshot',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'ResponseOfferSnapshot':
        return await self._unary_unary(
            '/tendermint.abci.ABCIApplication/OfferSnapshot',
            request_offer_snapshot,
            ResponseOfferSnapshot,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def load_snapshot_chunk(
        self,
        request_load_snapshot_chunk: 'RequestLoadSnapshotChunk',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'ResponseLoadSnapshotChunk':
        return await self._unary_unary(
            '/tendermint.abci.ABCIApplication/LoadSnapshotChunk',
            request_load_snapshot_chunk,
            ResponseLoadSnapshotChunk,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def apply_snapshot_chunk(
        self,
        request_apply_snapshot_chunk: 'RequestApplySnapshotChunk',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'ResponseApplySnapshotChunk':
        return await self._unary_unary(
            '/tendermint.abci.ABCIApplication/ApplySnapshotChunk',
            request_apply_snapshot_chunk,
            ResponseApplySnapshotChunk,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class AbciApplicationBase(ServiceBase):
    async def echo(self, request_echo: 'RequestEcho') -> 'ResponseEcho':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def flush(self, request_flush: 'RequestFlush') -> 'ResponseFlush':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def info(self, request_info: 'RequestInfo') -> 'ResponseInfo':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def set_option(
        self, request_set_option: 'RequestSetOption'
    ) -> 'ResponseSetOption':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def deliver_tx(
        self, request_deliver_tx: 'RequestDeliverTx'
    ) -> 'ResponseDeliverTx':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def check_tx(self, request_check_tx: 'RequestCheckTx') -> 'ResponseCheckTx':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def query(self, request_query: 'RequestQuery') -> 'ResponseQuery':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def commit(self, request_commit: 'RequestCommit') -> 'ResponseCommit':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def init_chain(
        self, request_init_chain: 'RequestInitChain'
    ) -> 'ResponseInitChain':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def begin_block(
        self, request_begin_block: 'RequestBeginBlock'
    ) -> 'ResponseBeginBlock':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def end_block(
        self, request_end_block: 'RequestEndBlock'
    ) -> 'ResponseEndBlock':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def list_snapshots(
        self, request_list_snapshots: 'RequestListSnapshots'
    ) -> 'ResponseListSnapshots':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def offer_snapshot(
        self, request_offer_snapshot: 'RequestOfferSnapshot'
    ) -> 'ResponseOfferSnapshot':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def load_snapshot_chunk(
        self, request_load_snapshot_chunk: 'RequestLoadSnapshotChunk'
    ) -> 'ResponseLoadSnapshotChunk':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def apply_snapshot_chunk(
        self, request_apply_snapshot_chunk: 'RequestApplySnapshotChunk'
    ) -> 'ResponseApplySnapshotChunk':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_echo(
        self, stream: 'grpclib.server.Stream[RequestEcho, ResponseEcho]'
    ) -> None:
        request = await stream.recv_message()
        response = await self.echo(request)
        await stream.send_message(response)

    async def __rpc_flush(
        self, stream: 'grpclib.server.Stream[RequestFlush, ResponseFlush]'
    ) -> None:
        request = await stream.recv_message()
        response = await self.flush(request)
        await stream.send_message(response)

    async def __rpc_info(
        self, stream: 'grpclib.server.Stream[RequestInfo, ResponseInfo]'
    ) -> None:
        request = await stream.recv_message()
        response = await self.info(request)
        await stream.send_message(response)

    async def __rpc_set_option(
        self, stream: 'grpclib.server.Stream[RequestSetOption, ResponseSetOption]'
    ) -> None:
        request = await stream.recv_message()
        response = await self.set_option(request)
        await stream.send_message(response)

    async def __rpc_deliver_tx(
        self, stream: 'grpclib.server.Stream[RequestDeliverTx, ResponseDeliverTx]'
    ) -> None:
        request = await stream.recv_message()
        response = await self.deliver_tx(request)
        await stream.send_message(response)

    async def __rpc_check_tx(
        self, stream: 'grpclib.server.Stream[RequestCheckTx, ResponseCheckTx]'
    ) -> None:
        request = await stream.recv_message()
        response = await self.check_tx(request)
        await stream.send_message(response)

    async def __rpc_query(
        self, stream: 'grpclib.server.Stream[RequestQuery, ResponseQuery]'
    ) -> None:
        request = await stream.recv_message()
        response = await self.query(request)
        await stream.send_message(response)

    async def __rpc_commit(
        self, stream: 'grpclib.server.Stream[RequestCommit, ResponseCommit]'
    ) -> None:
        request = await stream.recv_message()
        response = await self.commit(request)
        await stream.send_message(response)

    async def __rpc_init_chain(
        self, stream: 'grpclib.server.Stream[RequestInitChain, ResponseInitChain]'
    ) -> None:
        request = await stream.recv_message()
        response = await self.init_chain(request)
        await stream.send_message(response)

    async def __rpc_begin_block(
        self, stream: 'grpclib.server.Stream[RequestBeginBlock, ResponseBeginBlock]'
    ) -> None:
        request = await stream.recv_message()
        response = await self.begin_block(request)
        await stream.send_message(response)

    async def __rpc_end_block(
        self, stream: 'grpclib.server.Stream[RequestEndBlock, ResponseEndBlock]'
    ) -> None:
        request = await stream.recv_message()
        response = await self.end_block(request)
        await stream.send_message(response)

    async def __rpc_list_snapshots(
        self,
        stream: 'grpclib.server.Stream[RequestListSnapshots, ResponseListSnapshots]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.list_snapshots(request)
        await stream.send_message(response)

    async def __rpc_offer_snapshot(
        self,
        stream: 'grpclib.server.Stream[RequestOfferSnapshot, ResponseOfferSnapshot]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.offer_snapshot(request)
        await stream.send_message(response)

    async def __rpc_load_snapshot_chunk(
        self,
        stream: 'grpclib.server.Stream[RequestLoadSnapshotChunk, ResponseLoadSnapshotChunk]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.load_snapshot_chunk(request)
        await stream.send_message(response)

    async def __rpc_apply_snapshot_chunk(
        self,
        stream: 'grpclib.server.Stream[RequestApplySnapshotChunk, ResponseApplySnapshotChunk]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.apply_snapshot_chunk(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            '/tendermint.abci.ABCIApplication/Echo': grpclib.const.Handler(
                self.__rpc_echo,
                grpclib.const.Cardinality.UNARY_UNARY,
                RequestEcho,
                ResponseEcho,
            ),
            '/tendermint.abci.ABCIApplication/Flush': grpclib.const.Handler(
                self.__rpc_flush,
                grpclib.const.Cardinality.UNARY_UNARY,
                RequestFlush,
                ResponseFlush,
            ),
            '/tendermint.abci.ABCIApplication/Info': grpclib.const.Handler(
                self.__rpc_info,
                grpclib.const.Cardinality.UNARY_UNARY,
                RequestInfo,
                ResponseInfo,
            ),
            '/tendermint.abci.ABCIApplication/SetOption': grpclib.const.Handler(
                self.__rpc_set_option,
                grpclib.const.Cardinality.UNARY_UNARY,
                RequestSetOption,
                ResponseSetOption,
            ),
            '/tendermint.abci.ABCIApplication/DeliverTx': grpclib.const.Handler(
                self.__rpc_deliver_tx,
                grpclib.const.Cardinality.UNARY_UNARY,
                RequestDeliverTx,
                ResponseDeliverTx,
            ),
            '/tendermint.abci.ABCIApplication/CheckTx': grpclib.const.Handler(
                self.__rpc_check_tx,
                grpclib.const.Cardinality.UNARY_UNARY,
                RequestCheckTx,
                ResponseCheckTx,
            ),
            '/tendermint.abci.ABCIApplication/Query': grpclib.const.Handler(
                self.__rpc_query,
                grpclib.const.Cardinality.UNARY_UNARY,
                RequestQuery,
                ResponseQuery,
            ),
            '/tendermint.abci.ABCIApplication/Commit': grpclib.const.Handler(
                self.__rpc_commit,
                grpclib.const.Cardinality.UNARY_UNARY,
                RequestCommit,
                ResponseCommit,
            ),
            '/tendermint.abci.ABCIApplication/InitChain': grpclib.const.Handler(
                self.__rpc_init_chain,
                grpclib.const.Cardinality.UNARY_UNARY,
                RequestInitChain,
                ResponseInitChain,
            ),
            '/tendermint.abci.ABCIApplication/BeginBlock': grpclib.const.Handler(
                self.__rpc_begin_block,
                grpclib.const.Cardinality.UNARY_UNARY,
                RequestBeginBlock,
                ResponseBeginBlock,
            ),
            '/tendermint.abci.ABCIApplication/EndBlock': grpclib.const.Handler(
                self.__rpc_end_block,
                grpclib.const.Cardinality.UNARY_UNARY,
                RequestEndBlock,
                ResponseEndBlock,
            ),
            '/tendermint.abci.ABCIApplication/ListSnapshots': grpclib.const.Handler(
                self.__rpc_list_snapshots,
                grpclib.const.Cardinality.UNARY_UNARY,
                RequestListSnapshots,
                ResponseListSnapshots,
            ),
            '/tendermint.abci.ABCIApplication/OfferSnapshot': grpclib.const.Handler(
                self.__rpc_offer_snapshot,
                grpclib.const.Cardinality.UNARY_UNARY,
                RequestOfferSnapshot,
                ResponseOfferSnapshot,
            ),
            '/tendermint.abci.ABCIApplication/LoadSnapshotChunk': grpclib.const.Handler(
                self.__rpc_load_snapshot_chunk,
                grpclib.const.Cardinality.UNARY_UNARY,
                RequestLoadSnapshotChunk,
                ResponseLoadSnapshotChunk,
            ),
            '/tendermint.abci.ABCIApplication/ApplySnapshotChunk': grpclib.const.Handler(
                self.__rpc_apply_snapshot_chunk,
                grpclib.const.Cardinality.UNARY_UNARY,
                RequestApplySnapshotChunk,
                ResponseApplySnapshotChunk,
            ),
        }



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: evmos/recovery/v1/genesis.proto, evmos/recovery/v1/query.proto
# plugin: python-betterproto
# This file has been @generated
from dataclasses import dataclass
from datetime import timedelta
from typing import (
    TYPE_CHECKING,
    Dict,
    Optional,
)

import betterproto
import grpclib
from betterproto.grpc.grpclib_server import ServiceBase


if TYPE_CHECKING:
    import grpclib.server
    from betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class GenesisState(betterproto.Message):
    """GenesisState defines the recovery module's genesis state."""

    params: 'Params' = betterproto.message_field(1)
    """params defines all the paramaters of the module."""


@dataclass(eq=False, repr=False)
class Params(betterproto.Message):
    """Params holds parameters for the recovery module"""

    enable_recovery: bool = betterproto.bool_field(1)
    """enable recovery IBC middleware"""

    packet_timeout_duration: timedelta = betterproto.message_field(2)
    """
    duration added to timeout timestamp for balances recovered via IBC packets
    """


@dataclass(eq=False, repr=False)
class QueryParamsRequest(betterproto.Message):
    """
    QueryParamsRequest is the request type for the Query/Params RPC method.
    """

    pass


@dataclass(eq=False, repr=False)
class QueryParamsResponse(betterproto.Message):
    """
    QueryParamsResponse is the response type for the Query/Params RPC method.
    """

    params: 'Params' = betterproto.message_field(1)
    """params defines the parameters of the module."""


class QueryStub(betterproto.ServiceStub):
    async def params(
        self,
        query_params_request: 'QueryParamsRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryParamsResponse':
        return await self._unary_unary(
            '/evmos.recovery.v1.Query/Params',
            query_params_request,
            QueryParamsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class QueryBase(ServiceBase):
    async def params(
        self, query_params_request: 'QueryParamsRequest'
    ) -> 'QueryParamsResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_params(
        self, stream: 'grpclib.server.Stream[QueryParamsRequest, QueryParamsResponse]'
    ) -> None:
        request = await stream.recv_message()
        response = await self.params(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            '/evmos.recovery.v1.Query/Params': grpclib.const.Handler(
                self.__rpc_params,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryParamsRequest,
                QueryParamsResponse,
            ),
        }



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: evmos/erc20/v1/erc20.proto, evmos/erc20/v1/genesis.proto, evmos/erc20/v1/query.proto, evmos/erc20/v1/tx.proto
# plugin: python-betterproto
# This file has been @generated
from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    Dict,
    List,
    Optional,
)

import betterproto
import grpclib
from betterproto.grpc.grpclib_server import ServiceBase

from ....cosmos.bank import v1beta1 as ___cosmos_bank_v1_beta1__
from ....cosmos.base import v1beta1 as ___cosmos_base_v1_beta1__
from ....cosmos.base.query import v1beta1 as ___cosmos_base_query_v1_beta1__


if TYPE_CHECKING:
    import grpclib.server
    from betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


class Owner(betterproto.Enum):
    """Owner enumerates the ownership of a ERC20 contract."""

    OWNER_UNSPECIFIED = 0
    """OWNER_UNSPECIFIED defines an invalid/undefined owner."""

    OWNER_MODULE = 1
    """OWNER_MODULE erc20 is owned by the erc20 module account."""

    OWNER_EXTERNAL = 2
    """EXTERNAL erc20 is owned by an external account."""


@dataclass(eq=False, repr=False)
class TokenPair(betterproto.Message):
    """
    TokenPair defines an instance that records a pairing consisting of a native
     Cosmos Coin and an ERC20 token address.
    """

    erc20_address: str = betterproto.string_field(1)
    """address of ERC20 contract token"""

    denom: str = betterproto.string_field(2)
    """cosmos base denomination to be mapped to"""

    enabled: bool = betterproto.bool_field(3)
    """shows token mapping enable status"""

    contract_owner: 'Owner' = betterproto.enum_field(4)
    """
    ERC20 owner address ENUM (0 invalid, 1 ModuleAccount, 2 external address)
    """


@dataclass(eq=False, repr=False)
class RegisterCoinProposal(betterproto.Message):
    """
    RegisterCoinProposal is a gov Content type to register a token pair for a
    native Cosmos coin.
    """

    title: str = betterproto.string_field(1)
    """title of the proposal"""

    description: str = betterproto.string_field(2)
    """proposal description"""

    metadata: '___cosmos_bank_v1_beta1__.Metadata' = betterproto.message_field(3)
    """metadata of the native Cosmos coin"""


@dataclass(eq=False, repr=False)
class RegisterErc20Proposal(betterproto.Message):
    """
    RegisterERC20Proposal is a gov Content type to register a token pair for an
    ERC20 token
    """

    title: str = betterproto.string_field(1)
    """title of the proposal"""

    description: str = betterproto.string_field(2)
    """proposal description"""

    erc20_address: str = betterproto.string_field(3)
    """contract address of ERC20 token"""


@dataclass(eq=False, repr=False)
class ToggleTokenConversionProposal(betterproto.Message):
    """
    ToggleTokenConversionProposal is a gov Content type to toggle the conversion
    of a token pair.
    """

    title: str = betterproto.string_field(1)
    """title of the proposal"""

    description: str = betterproto.string_field(2)
    """proposal description"""

    token: str = betterproto.string_field(3)
    """
    token identifier can be either the hex contract address of the ERC20 or the
    Cosmos base denomination
    """


@dataclass(eq=False, repr=False)
class GenesisState(betterproto.Message):
    """GenesisState defines the module's genesis state."""

    params: 'Params' = betterproto.message_field(1)
    """module parameters"""

    token_pairs: List['TokenPair'] = betterproto.message_field(2)
    """registered token pairs"""


@dataclass(eq=False, repr=False)
class Params(betterproto.Message):
    """Params defines the erc20 module params"""

    enable_erc20: bool = betterproto.bool_field(1)
    """
    parameter to enable the conversion of Cosmos coins <--> ERC20 tokens.
    """

    enable_evm_hook: bool = betterproto.bool_field(2)
    """
    parameter to enable the EVM hook that converts an ERC20 token to a Cosmos
    Coin by transferring the Tokens through a MsgEthereumTx to the
    ModuleAddress Ethereum address.
    """


@dataclass(eq=False, repr=False)
class QueryTokenPairsRequest(betterproto.Message):
    """
    QueryTokenPairsRequest is the request type for the Query/TokenPairs RPC
    method.
    """

    pagination: '___cosmos_base_query_v1_beta1__.PageRequest' = (
        betterproto.message_field(1)
    )
    """pagination defines an optional pagination for the request."""


@dataclass(eq=False, repr=False)
class QueryTokenPairsResponse(betterproto.Message):
    """
    QueryTokenPairsResponse is the response type for the Query/TokenPairs RPC
    method.
    """

    token_pairs: List['TokenPair'] = betterproto.message_field(1)
    pagination: '___cosmos_base_query_v1_beta1__.PageResponse' = (
        betterproto.message_field(2)
    )
    """pagination defines the pagination in the response."""


@dataclass(eq=False, repr=False)
class QueryTokenPairRequest(betterproto.Message):
    """
    QueryTokenPairRequest is the request type for the Query/TokenPair RPC method.
    """

    token: str = betterproto.string_field(1)
    """
    token identifier can be either the hex contract address of the ERC20 or the
    Cosmos base denomination
    """


@dataclass(eq=False, repr=False)
class QueryTokenPairResponse(betterproto.Message):
    """
    QueryTokenPairResponse is the response type for the Query/TokenPair RPC
    method.
    """

    token_pair: 'TokenPair' = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class QueryParamsRequest(betterproto.Message):
    """
    QueryParamsRequest is the request type for the Query/Params RPC method.
    """

    pass


@dataclass(eq=False, repr=False)
class QueryParamsResponse(betterproto.Message):
    """
    QueryParamsResponse is the response type for the Query/Params RPC
    method.
    """

    params: 'Params' = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class MsgConvertCoin(betterproto.Message):
    """
    MsgConvertCoin defines a Msg to convert a native Cosmos coin to a ERC20 token
    """

    coin: '___cosmos_base_v1_beta1__.Coin' = betterproto.message_field(1)
    """
    Cosmos coin which denomination is registered in a token pair. The coin
    amount defines the amount of coins to convert.
    """

    receiver: str = betterproto.string_field(2)
    """recipient hex address to receive ERC20 token"""

    sender: str = betterproto.string_field(3)
    """cosmos bech32 address from the owner of the given Cosmos coins"""


@dataclass(eq=False, repr=False)
class MsgConvertCoinResponse(betterproto.Message):
    """MsgConvertCoinResponse returns no fields"""

    pass


@dataclass(eq=False, repr=False)
class MsgConvertErc20(betterproto.Message):
    """
    MsgConvertERC20 defines a Msg to convert a ERC20 token to a native Cosmos
    coin.
    """

    contract_address: str = betterproto.string_field(1)
    """ERC20 token contract address registered in a token pair"""

    amount: str = betterproto.string_field(2)
    """amount of ERC20 tokens to convert"""

    receiver: str = betterproto.string_field(3)
    """bech32 address to receive native Cosmos coins"""

    sender: str = betterproto.string_field(4)
    """sender hex address from the owner of the given ERC20 tokens"""


@dataclass(eq=False, repr=False)
class MsgConvertErc20Response(betterproto.Message):
    """MsgConvertERC20Response returns no fields"""

    pass


class QueryStub(betterproto.ServiceStub):
    async def token_pairs(
        self,
        query_token_pairs_request: 'QueryTokenPairsRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryTokenPairsResponse':
        return await self._unary_unary(
            '/evmos.erc20.v1.Query/TokenPairs',
            query_token_pairs_request,
            QueryTokenPairsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def token_pair(
        self,
        query_token_pair_request: 'QueryTokenPairRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryTokenPairResponse':
        return await self._unary_unary(
            '/evmos.erc20.v1.Query/TokenPair',
            query_token_pair_request,
            QueryTokenPairResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def params(
        self,
        query_params_request: 'QueryParamsRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryParamsResponse':
        return await self._unary_unary(
            '/evmos.erc20.v1.Query/Params',
            query_params_request,
            QueryParamsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class MsgStub(betterproto.ServiceStub):
    async def convert_coin(
        self,
        msg_convert_coin: 'MsgConvertCoin',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'MsgConvertCoinResponse':
        return await self._unary_unary(
            '/evmos.erc20.v1.Msg/ConvertCoin',
            msg_convert_coin,
            MsgConvertCoinResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def convert_erc20(
        self,
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'MsgConvertErc20Response':
        return await self._unary_unary(
            '/evmos.erc20.v1.Msg/ConvertERC20',
            msg_convert_erc20,
            MsgConvertErc20Response,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class QueryBase(ServiceBase):
    async def token_pairs(
        self, query_token_pairs_request: 'QueryTokenPairsRequest'
    ) -> 'QueryTokenPairsResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def token_pair(
        self, query_token_pair_request: 'QueryTokenPairRequest'
    ) -> 'QueryTokenPairResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def params(
        self, query_params_request: 'QueryParamsRequest'
    ) -> 'QueryParamsResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_token_pairs(
        self,
        stream: 'grpclib.server.Stream[QueryTokenPairsRequest, QueryTokenPairsResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.token_pairs(request)
        await stream.send_message(response)

    async def __rpc_token_pair(
        self,
        stream: 'grpclib.server.Stream[QueryTokenPairRequest, QueryTokenPairResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.token_pair(request)
        await stream.send_message(response)

    async def __rpc_params(
        self, stream: 'grpclib.server.Stream[QueryParamsRequest, QueryParamsResponse]'
    ) -> None:
        request = await stream.recv_message()
        response = await self.params(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            '/evmos.erc20.v1.Query/TokenPairs': grpclib.const.Handler(
                self.__rpc_token_pairs,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryTokenPairsRequest,
                QueryTokenPairsResponse,
            ),
            '/evmos.erc20.v1.Query/TokenPair': grpclib.const.Handler(
                self.__rpc_token_pair,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryTokenPairRequest,
                QueryTokenPairResponse,
            ),
            '/evmos.erc20.v1.Query/Params': grpclib.const.Handler(
                self.__rpc_params,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryParamsRequest,
                QueryParamsResponse,
            ),
        }


class MsgBase(ServiceBase):
    async def convert_coin(
        self, msg_convert_coin: 'MsgConvertCoin'
    ) -> 'MsgConvertCoinResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def convert_erc20(self) -> 'MsgConvertErc20Response':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_convert_coin(
        self, stream: 'grpclib.server.Stream[MsgConvertCoin, MsgConvertCoinResponse]'
    ) -> None:
        request = await stream.recv_message()
        response = await self.convert_coin(request)
        await stream.send_message(response)

    async def __rpc_convert_erc20(
        self, stream: 'grpclib.server.Stream[MsgConvertErc20, MsgConvertErc20Response]'
    ) -> None:
        request = await stream.recv_message()
        response = await self.convert_erc20(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            '/evmos.erc20.v1.Msg/ConvertCoin': grpclib.const.Handler(
                self.__rpc_convert_coin,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgConvertCoin,
                MsgConvertCoinResponse,
            ),
            '/evmos.erc20.v1.Msg/ConvertERC20': grpclib.const.Handler(
                self.__rpc_convert_erc20,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgConvertErc20,
                MsgConvertErc20Response,
            ),
        }



from __future__ import annotations

from typing import Final

from evmos.eip712.base import MsgInterface, MsgWithValidatorInterface

MSG_DELEGATE_TYPES: Final = {
    'MsgValue': [
        {'name': 'delegator_address', 'type': 'string'},
        {'name': 'validator_address', 'type': 'string'},
        {'name': 'amount', 'type': 'TypeAmount'},
    ],
    'TypeAmount': [
        {'name': 'denom', 'type': 'string'},
        {'name': 'amount', 'type': 'string'},
    ],
}
"""Types for delegation message."""


def create_msg_delegate(
    delegator_address: str,
    validator_address: str,
    amount: str,
    denom: str,
) -> MsgInterface:
    """Create delegation message."""
    return {
        'type': 'cosmos-sdk/MsgDelegate',
        'value': {
            'amount': {'amount': amount, 'denom': denom},
            'delegator_address': delegator_address,
            'validator_address': validator_address,
        },
    }


MSG_BEGIN_REDELEGATE_TYPES: Final = {
    'MsgValue': [
        {'name': 'delegator_address', 'type': 'string'},
        {'name': 'validator_src_address', 'type': 'string'},
        {'name': 'validator_dst_address', 'type': 'string'},
        {'name': 'amount', 'type': 'TypeAmount'},
    ],
    'TypeAmount': [
        {'name': 'denom', 'type': 'string'},
        {'name': 'amount', 'type': 'string'},
    ],
}
"""Types for redelegation beginning message."""


def create_msg_begin_redelegate(
    delegator_address: str,
    validator_src_address: str,
    validator_dst_address: str,
    amount: str,
    denom: str,
) -> MsgInterface:
    """Create redelegation beginning message."""
    return {
        'type': 'cosmos-sdk/MsgBeginRedelegate',
        'value': {
            'amount': {'amount': amount, 'denom': denom},
            'delegator_address': delegator_address,
            'validator_src_address': validator_src_address,
            'validator_dst_address': validator_dst_address,
        },
    }


MSG_UNDELEGATE_TYPES: Final = {
    'MsgValue': [
        {'name': 'delegator_address', 'type': 'string'},
        {'name': 'validator_address', 'type': 'string'},
        {'name': 'amount', 'type': 'TypeAmount'},
    ],
    'TypeAmount': [
        {'name': 'denom', 'type': 'string'},
        {'name': 'amount', 'type': 'string'},
    ],
}
"""Types for delegation cancellation message."""


def create_msg_undelegate(
    delegator_address: str,
    validator_address: str,
    amount: str,
    denom: str,
) -> MsgInterface:
    """Create delegation cancellation message."""
    return {
        'type': 'cosmos-sdk/MsgUndelegate',
        'value': {
            'amount': {'amount': amount, 'denom': denom},
            'delegator_address': delegator_address,
            'validator_address': validator_address,
        },
    }


MSG_WITHDRAW_DELEGATOR_REWARD_TYPES: Final = {
    'MsgValue': [
        {'name': 'delegator_address', 'type': 'string'},
        {'name': 'validator_address', 'type': 'string'},
    ],
}
"""Types for delegation reward withdrawal message."""


def create_msg_withdraw_delegator_reward(
    delegator_address: str,
    validator_address: str,
) -> MsgInterface:
    """Create delegation reward withdrawal message."""
    return {
        'type': 'cosmos-sdk/MsgWithdrawDelegationReward',
        'value': {
            'delegator_address': delegator_address,
            'validator_address': validator_address,
        },
    }


MSG_WITHDRAW_VALIDATOR_COMMISSION_TYPES: Final = {
    'MsgValue': [{'name': 'validator_address', 'type': 'string'}],
}
"""Types for validator commission withdrawal message."""


def create_msg_withdraw_validator_commission(
    validator_address: str,
) -> MsgWithValidatorInterface:
    """Create validator commission withdrawal message."""
    return {
        'type': 'cosmos-sdk/MsgWithdrawValidatorCommission',
        'value': {'validator_address': validator_address},
    }



from __future__ import annotations

from dataclasses import dataclass
from typing import Generic, TypeVar

from betterproto import Message
from betterproto.lib.google.protobuf import Any as GoogleAny

_M = TypeVar('_M', bound=Message)


@dataclass
class MessageGenerated(Generic[_M]):
    """Structure of message generated by our library."""

    message: _M
    """Actual message instance."""
    path: str
    """Path where this was generated inside the library."""


def create_any_message(msg: MessageGenerated[Message]) -> GoogleAny:
    """Wrap message (coerced to binary) with convenience wrapper.

    Returns:
        :class:`~betterproto.lib.google.protobuf.Any`
    """
    return GoogleAny(
        type_url=f'/{msg.path}',
        value=bytes(msg.message),
    )



from __future__ import annotations

from typing import Final

from evmos.eip712.base import (
    Domain,
    EIPToSign,
    MsgInterface,
    create_eip712,
    generate_fee,
    generate_message,
    generate_message_with_multiple_transactions,
    generate_types,
)
from evmos.eip712.erc20 import (
    MSG_CONVERT_COIN_TYPES,
    MSG_CONVERT_ERC20_TYPES,
    create_msg_convert_coin,
    create_msg_convert_erc20,
)
from evmos.eip712.feesplit import (
    MSG_CANCEL_FEE_SPLIT_TYPES,
    MSG_REGISTER_FEE_SPLIT_TYPES,
    MSG_UPDATE_FEE_SPLIT_TYPES,
    create_msg_cancel_fee_split,
    create_msg_register_fee_split,
    create_msg_update_fee_split,
)
from evmos.eip712.staking import (
    MSG_BEGIN_REDELEGATE_TYPES,
    MSG_DELEGATE_TYPES,
    MSG_UNDELEGATE_TYPES,
    MSG_WITHDRAW_DELEGATOR_REWARD_TYPES,
    MSG_WITHDRAW_VALIDATOR_COMMISSION_TYPES,
    create_msg_begin_redelegate,
    create_msg_delegate,
    create_msg_undelegate,
    create_msg_withdraw_delegator_reward,
    create_msg_withdraw_validator_commission,
)
from evmos.eip712.validator import MSG_EDIT_VALIDATOR_TYPES, create_msg_edit_validator

__all__ = [
    'create_eip712',
    'create_ibc_msg_transfer',
    'create_msg_begin_redelegate',
    'create_msg_cancel_fee_split',
    'create_msg_convert_coin',
    'create_msg_convert_erc20',
    'MSG_CANCEL_FEE_SPLIT_TYPES',
    'create_msg_delegate',
    'create_msg_edit_validator',
    'MSG_VOTE_TYPES',
    'create_msg_register_fee_split',
    'create_msg_send',
    'create_msg_undelegate',
    'create_msg_update_fee_split',
    'MSG_DELEGATE_TYPES',
    'create_msg_vote',
    'create_msg_withdraw_delegator_reward',
    'create_msg_withdraw_validator_commission',
    'generate_fee',
    'MSG_CONVERT_COIN_TYPES',
    'generate_message',
    'generate_message_with_multiple_transactions',
    'generate_types',
    'IBC_MSG_TRANSFER_TYPES',
    'MSG_BEGIN_REDELEGATE_TYPES',
    'MSG_CONVERT_ERC20_TYPES',
    'MSG_REGISTER_FEE_SPLIT_TYPES',
    'MSG_SEND_TYPES',
    'MSG_UNDELEGATE_TYPES',
    'MSG_UPDATE_FEE_SPLIT_TYPES',
    'MSG_WITHDRAW_DELEGATOR_REWARD_TYPES',
    'MSG_WITHDRAW_VALIDATOR_COMMISSION_TYPES',
    'MSG_EDIT_VALIDATOR_TYPES',
    'EIPToSign',
    'Domain',
]


# gov.ts
MSG_VOTE_TYPES: Final = {
    'MsgValue': [
        {'name': 'proposalId', 'type': 'uint64'},
        {'name': 'voter', 'type': 'string'},
        {'name': 'option', 'type': 'int32'},
    ],
}
"""Types for voting message."""


def create_msg_vote(proposal_id: int, option: int, sender: str) -> MsgInterface:
    """Create voting (governmental) message."""
    return {
        'type': 'cosmos-sdk/MsgVote',
        'value': {
            'proposalId': proposal_id,
            'voter': sender,
            'option': option,
        },
    }


# ibcMsgTransfer.ts
IBC_MSG_TRANSFER_TYPES: Final = {
    'MsgValue': [
        {'name': 'sourcePort', 'type': 'string'},
        {'name': 'sourceChannel', 'type': 'string'},
        {'name': 'token', 'type': 'TypeToken'},
        {'name': 'sender', 'type': 'string'},
        {'name': 'receiver', 'type': 'string'},
        {'name': 'timeoutHeight', 'type': 'TypeTimeoutHeight'},
        {'name': 'timeoutTimestamp', 'type': 'uint64'},
    ],
    'TypeToken': [
        {'name': 'denom', 'type': 'string'},
        {'name': 'amount', 'type': 'string'},
    ],
    'TypeTimeoutHeight': [
        {'name': 'revisionNumber', 'type': 'uint64'},
        {'name': 'revisionHeight', 'type': 'uint64'},
    ],
}
"""Types for IBC message."""


def create_ibc_msg_transfer(
    receiver: str,
    sender: str,
    source_channel: str,
    source_port: str,
    revision_height: int,
    revision_number: int,
    timeout_timestamp: str,
    amount: str,
    denom: str,
) -> MsgInterface:
    """Create IBC (inter-blockchain) message transfer.

    See Also:
        https://tutorials.cosmos.network/academy/4-ibc/what-is-ibc.html
    """
    return {
        'type': 'cosmos-sdk/MsgTransfer',
        'value': {
            'receiver': receiver,
            'sender': sender,
            'sourceChannel': source_channel,
            'sourcePort': source_port,
            'timeoutHeight': {
                'revisionHeight': str(revision_height),
                'revisionNumber': str(revision_number),
            },
            'timeoutTimestamp': timeout_timestamp,
            'token': {'amount': amount, 'denom': denom},
        },
    }


# msgSend.ts
MSG_SEND_TYPES: Final = {
    'MsgValue': [
        {'name': 'from_address', 'type': 'string'},
        {'name': 'to_address', 'type': 'string'},
        {'name': 'amount', 'type': 'TypeAmount[]'},
    ],
    'TypeAmount': [
        {'name': 'denom', 'type': 'string'},
        {'name': 'amount', 'type': 'string'},
    ],
}
"""Types for message sending."""


def create_msg_send(
    amount: str,
    denom: str,
    from_address: str,
    to_address: str,
) -> MsgInterface:
    """Create message for sending."""
    return {
        'type': 'cosmos-sdk/MsgSend',
        'value': {
            'amount': [
                {'amount': amount, 'denom': denom},
            ],
            'from_address': from_address,
            'to_address': to_address,
        },
    }



"""Hashing utilities for EIP-712 messages.

Copied under the MIT license from
https://github.com/ethereum/eth-account/blob/master/eth_account/_utils/structured_data/hashing.py
with minor modifications.
"""
from __future__ import annotations

from dataclasses import asdict
from itertools import chain, groupby
from operator import itemgetter
from typing import Any, Iterable, Iterator, Mapping, Sequence, TypedDict

from eth_abi import encode, is_encodable, is_encodable_type
from eth_abi.grammar import parse
from eth_utils import keccak, to_tuple
from typing_extensions import TypeAlias

from evmos.eip712 import EIPToSign


class _FieldT(TypedDict):
    name: str
    type: str  # noqa: A003


_TypesT: TypeAlias = Mapping[str, Sequence[_FieldT]]


def get_dependencies(primary_type: str, types: _TypesT) -> tuple[str, ...]:
    """Perform DFS to get all the dependencies of the `primary_type`."""
    deps = set()
    struct_names_yet_to_be_expanded = [primary_type]

    while struct_names_yet_to_be_expanded:
        struct_name = struct_names_yet_to_be_expanded.pop()

        deps.add(struct_name)
        fields = types[struct_name]
        for field in fields:
            field_type = field['type']

            # Handle array types
            if is_array_type(field_type):
                field_type = field_type[: field_type.index('[')]

            if field_type in types and field_type not in deps:
                # We don't need to expand types that are not user defined (customized)
                # Also skip types that we have already encountered
                struct_names_yet_to_be_expanded.append(field_type)

    # Don't need to make a struct as dependency of itself
    deps.remove(primary_type)

    return tuple(deps)


def field_identifier(field: _FieldT) -> str:
    """Stringify a field in ``'TYPE NAME'`` format."""
    return f'{field["type"]} {field["name"]}'


def encode_struct(struct_name: str, struct_field_types: Iterable[_FieldT]) -> str:
    """Stringify a single struct in ``'NAME(type1 name1,type2 name2,...)'`` format."""
    return '{name}({args})'.format(
        name=struct_name,
        args=','.join(map(field_identifier, struct_field_types)),
    )


def encode_type(primary_type: str, types: _TypesT) -> str:
    """Encode type as concatenation of itself and all dependencies (alphabetical order).

    The type of a struct is encoded as

    .. code-block:: text

        name ‖ "(" ‖ member₁ ‖ "," ‖ member₂ ‖ "," ‖ … ‖ memberₙ ")"

    where each member is written as ``type ‖ " " ‖ name``.
    """
    # Getting the dependencies and sorting them alphabetically as per EIP712
    deps = get_dependencies(primary_type, types)

    return ''.join(
        encode_struct(struct_name, types[struct_name])
        for struct_name in chain((primary_type,), sorted(deps))
    )


def hash_struct_type(primary_type: str, types: _TypesT) -> bytes:
    """Hash string representation of type of struct."""
    return keccak(text=encode_type(primary_type, types))


def is_array_type(type_: str) -> bool:
    """Identify if type such as ``person[]`` or ``person[2]`` is an array."""
    return type_.endswith(']')


@to_tuple
def get_depths_and_dimensions(data: Any, depth: int) -> Iterator[tuple[int, int]]:
    """Generate tuples of depth and dimension of each element at that depth."""
    if not isinstance(data, (list, tuple)):
        # Not checking for Iterable instance, because even Dictionaries and strings
        # are considered as iterables, but that's not what we want the condition to be.
        return ()

    yield depth, len(data)

    for item in data:
        # iterating over all 1 dimension less sub-data items
        yield from get_depths_and_dimensions(item, depth + 1)


def get_array_dimensions(data: Any) -> tuple[int | str, ...]:
    """Given an data item, check that it is an array and return the dimensions.

    Examples:
        >>> get_array_dimensions([[1, 2, 3], [4, 5, 6]])
        (3, 2)

    """
    depths_and_dimensions = get_depths_and_dimensions(data, 0)
    # re-form as a dictionary with `depth` as key, and all of the dimensions
    # found at that depth.
    grouped_by_depth = {
        depth: tuple(dimension for depth, dimension in group)
        for depth, group in groupby(depths_and_dimensions, itemgetter(0))
    }

    return tuple(
        # check that all dimensions are the same, else use "dynamic"
        dimensions[0] if all(dim == dimensions[0] for dim in dimensions) else 'dynamic'
        for _depth, dimensions in sorted(grouped_by_depth.items(), reverse=True)
    )


def _check_dimensions(field_type: str, value: Any) -> None:
    # Get the dimensions from the value
    array_dimensions = get_array_dimensions(value)
    # Get the dimensions from what was declared in the schema
    parsed_field_type = parse(field_type)

    for given, expected in zip(array_dimensions, parsed_field_type.arrlist):
        if not expected:
            # Skip empty or dynamically declared dimensions
            continue
        if given != expected[0]:
            # Dimensions should match with declared schema
            expected_dimensions_repr = tuple(
                map(lambda x: x[0] if x else 'dynamic', parsed_field_type.arrlist)
            )
            raise TypeError(
                f'Array data `{value}` has dimensions `{array_dimensions}`'
                f' whereas the schema has dimensions `{expected_dimensions_repr}`'
            )


def _encode_array_field(
    types: _TypesT, name: str, field_type: str, value: Any
) -> bytes:
    _check_dimensions(field_type, value)

    if value:
        field_type_of_inside_array = field_type[: field_type.rindex('[')]
        field_type_value_pairs = [
            encode_field(types, name, field_type_of_inside_array, item)
            for item in value
        ]

        data_types, data_hashes = zip(*field_type_value_pairs)
    else:
        data_types = data_hashes = tuple()

    return keccak(encode(data_types, data_hashes))


def encode_field(
    types: _TypesT, name: str, field_type: str, value: Any
) -> tuple[str, bytes]:
    """Encode field according to given type.

    Args:
        types: mapping with all known types
        name: field name
        field_type: type of field (must be present in ``types`` or be basic type)
        value: value to encode

    Returns:
        tuple of form (result_type, encoded_bytes)
    """
    if value is None:
        raise ValueError(f'Missing value for field {name} of type {field_type}')

    if field_type in types:
        return ('bytes32', keccak(encode_data(field_type, types, value)))

    if field_type == 'bytes':
        if not isinstance(value, bytes):
            raise TypeError(
                f'Value of field `{name}` ({value}) is of the type `{type(value)}`, '
                f'but expected bytes value'
            )

        return ('bytes32', keccak(value))

    if field_type == 'string':
        if not isinstance(value, str):
            raise TypeError(
                f'Value of field `{name}` ({value}) is of the type `{type(value)}`, '
                f'but expected string value'
            )

        return ('bytes32', keccak(text=value))

    if is_array_type(field_type):
        encoded = _encode_array_field(types, name, field_type, value)
        return ('bytes32', encoded)

    # First checking to see if field_type is valid as per abi
    if not is_encodable_type(field_type):
        raise TypeError(f'Received Invalid type `{field_type}` in field `{name}`')

    # Next, see if the value is encodable as the specified field_type
    if is_encodable(field_type, value):
        # field_type is a valid type and the provided value is encodable as that type
        return (field_type, value)

    raise TypeError(
        f'Value of `{name}` ({value}) is not encodable as type `{field_type}`. '
        f'If the base type is correct, verify that the value does not '
        f'exceed the specified size for the type.'
    )


def encode_data(primary_type: str, types: _TypesT, data: Any) -> bytes:
    """Encode data defined by ``primary_type``."""
    encoded_types = ['bytes32']
    encoded_values = [hash_struct_type(primary_type, types)]

    for field in types[primary_type]:
        type_, value = encode_field(
            types, field['name'], field['type'], data[field['name']]
        )
        encoded_types.append(type_)
        encoded_values.append(value)

    return encode(encoded_types, encoded_values)


def hash_domain(structured_data: EIPToSign) -> bytes:
    """Hash domain part of EIP-712 message."""
    return keccak(
        encode_data(
            'EIP712Domain', structured_data.types, asdict(structured_data.domain)
        )
    )


def hash_message(structured_data: EIPToSign) -> bytes:
    """Hash message part of EIP-712 message."""
    return keccak(
        encode_data(
            structured_data.primaryType,
            structured_data.types,
            structured_data.message,
        )
    )



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: proofs.proto
# plugin: python-betterproto
# This file has been @generated
from dataclasses import dataclass
from typing import List

import betterproto


class HashOp(betterproto.Enum):
    NO_HASH = 0
    """
    NO_HASH is the default if no data passed. Note this is an illegal argument
    some places.
    """

    SHA256 = 1
    SHA512 = 2
    KECCAK = 3
    RIPEMD160 = 4
    BITCOIN = 5


class LengthOp(betterproto.Enum):
    """
    *
    LengthOp defines how to process the key and value of the LeafOp
    to include length information. After encoding the length with the given
    algorithm, the length will be prepended to the key and value bytes.
    (Each one with it's own encoded length)
    """

    NO_PREFIX = 0
    """NO_PREFIX don't include any length info"""

    VAR_PROTO = 1
    """VAR_PROTO uses protobuf (and go-amino) varint encoding of the length"""

    VAR_RLP = 2
    """VAR_RLP uses rlp int encoding of the length"""

    FIXED32_BIG = 3
    """
    FIXED32_BIG uses big-endian encoding of the length as a 32 bit integer
    """

    FIXED32_LITTLE = 4
    """
    FIXED32_LITTLE uses little-endian encoding of the length as a 32 bit
    integer
    """

    FIXED64_BIG = 5
    """
    FIXED64_BIG uses big-endian encoding of the length as a 64 bit integer
    """

    FIXED64_LITTLE = 6
    """
    FIXED64_LITTLE uses little-endian encoding of the length as a 64 bit
    integer
    """

    REQUIRE_32_BYTES = 7
    """
    REQUIRE_32_BYTES is like NONE, but will fail if the input is not exactly 32
    bytes (sha256 output)
    """

    REQUIRE_64_BYTES = 8
    """
    REQUIRE_64_BYTES is like NONE, but will fail if the input is not exactly 64
    bytes (sha512 output)
    """


@dataclass(eq=False, repr=False)
class ExistenceProof(betterproto.Message):
    """
    *
    ExistenceProof takes a key and a value and a set of steps to perform on it.
    The result of peforming all these steps will provide a "root hash", which can
    be compared to the value in a header.
    Since it is computationally infeasible to produce a hash collission for any of
    the used cryptographic hash functions, if someone can provide a series of
    operations to transform a given key and value into a root hash that matches some
    trusted root, these key and values must be in the referenced merkle tree.
    The only possible issue is maliablity in LeafOp, such as providing extra prefix
    data, which should be controlled by a spec. Eg. with lengthOp as NONE, prefix =
    FOO, key = BAR, value = CHOICE and prefix = F, key = OOBAR, value = CHOICE would
    produce the same value.
    With LengthOp this is tricker but not impossible. Which is why the
    "leafPrefixEqual" field in the ProofSpec is valuable to prevent this mutability.
    And why all trees should length-prefix the data before hashing it.
    """

    key: bytes = betterproto.bytes_field(1)
    value: bytes = betterproto.bytes_field(2)
    leaf: 'LeafOp' = betterproto.message_field(3)
    path: List['InnerOp'] = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class NonExistenceProof(betterproto.Message):
    """
    NonExistenceProof takes a proof of two neighbors, one left of the desired key,
    one right of the desired key. If both proofs are valid AND they are neighbors,
    then there is no valid proof for the given key.
    """

    key: bytes = betterproto.bytes_field(1)
    left: 'ExistenceProof' = betterproto.message_field(2)
    right: 'ExistenceProof' = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class CommitmentProof(betterproto.Message):
    """
    CommitmentProof is either an ExistenceProof or a NonExistenceProof, or a Batch
    of such messages
    """

    exist: 'ExistenceProof' = betterproto.message_field(1, group='proof')
    nonexist: 'NonExistenceProof' = betterproto.message_field(2, group='proof')
    batch: 'BatchProof' = betterproto.message_field(3, group='proof')
    compressed: 'CompressedBatchProof' = betterproto.message_field(4, group='proof')


@dataclass(eq=False, repr=False)
class LeafOp(betterproto.Message):
    """
    *
    LeafOp represents the raw key-value data we wish to prove, and
    must be flexible to represent the internal transformation from
    the original key-value pairs into the basis hash, for many existing
    merkle trees.
    key and value are passed in. So that the signature of this operation is:
    leafOp(key, value) -> output
    To process this, first prehash the keys and values if needed (ANY means no hash
    in this case): hkey = prehashKey(key) hvalue = prehashValue(value)
    Then combine the bytes, and hash it
    output = hash(prefix || length(hkey) || hkey || length(hvalue) || hvalue)
    """

    hash: 'HashOp' = betterproto.enum_field(1)
    prehash_key: 'HashOp' = betterproto.enum_field(2)
    prehash_value: 'HashOp' = betterproto.enum_field(3)
    length: 'LengthOp' = betterproto.enum_field(4)
    prefix: bytes = betterproto.bytes_field(5)
    """
    prefix is a fixed bytes that may optionally be included at the beginning to
    differentiate a leaf node from an inner node.
    """


@dataclass(eq=False, repr=False)
class InnerOp(betterproto.Message):
    """
    *
    InnerOp represents a merkle-proof step that is not a leaf.
    It represents concatenating two children and hashing them to provide the next
    result.
    The result of the previous step is passed in, so the signature of this op is:
    innerOp(child) -> output
    The result of applying InnerOp should be:
    output = op.hash(op.prefix || child || op.suffix)
    where the || operator is concatenation of binary data,
    and child is the result of hashing all the tree below this step.
    Any special data, like prepending child with the length, or prepending the
    entire operation with some value to differentiate from leaf nodes, should be
    included in prefix and suffix. If either of prefix or suffix is empty, we just
    treat it as an empty string
    """

    hash: 'HashOp' = betterproto.enum_field(1)
    prefix: bytes = betterproto.bytes_field(2)
    suffix: bytes = betterproto.bytes_field(3)


@dataclass(eq=False, repr=False)
class ProofSpec(betterproto.Message):
    """
    *
    ProofSpec defines what the expected parameters are for a given proof type.
    This can be stored in the client and used to validate any incoming proofs.
    verify(ProofSpec, Proof) -> Proof | Error
    As demonstrated in tests, if we don't fix the algorithm used to calculate the
    LeafHash for a given tree, there are many possible key-value pairs that can
    generate a given hash (by interpretting the preimage differently).
    We need this for proper security, requires client knows a priori what
    tree format server uses. But not in code, rather a configuration object.
    """

    leaf_spec: 'LeafOp' = betterproto.message_field(1)
    """
    any field in the ExistenceProof must be the same as in this spec.
    except Prefix, which is just the first bytes of prefix (spec can be longer)
    """

    inner_spec: 'InnerSpec' = betterproto.message_field(2)
    max_depth: int = betterproto.int32_field(3)
    """
    max_depth (if > 0) is the maximum number of InnerOps allowed (mainly for
    fixed-depth tries)
    """

    min_depth: int = betterproto.int32_field(4)
    """
    min_depth (if > 0) is the minimum number of InnerOps allowed (mainly for
    fixed-depth tries)
    """


@dataclass(eq=False, repr=False)
class InnerSpec(betterproto.Message):
    """
    InnerSpec contains all store-specific structure info to determine if two proofs
    from a given store are neighbors.
    This enables:
    isLeftMost(spec: InnerSpec, op: InnerOp)
    isRightMost(spec: InnerSpec, op: InnerOp)
    isLeftNeighbor(spec: InnerSpec, left: InnerOp, right: InnerOp)
    """

    child_order: List[int] = betterproto.int32_field(1)
    """
    Child order is the ordering of the children node, must count from 0
    iavl tree is [0, 1] (left then right)
    merk is [0, 2, 1] (left, right, here)
    """

    child_size: int = betterproto.int32_field(2)
    min_prefix_length: int = betterproto.int32_field(3)
    max_prefix_length: int = betterproto.int32_field(4)
    empty_child: bytes = betterproto.bytes_field(5)
    """
    empty child is the prehash image that is used when one child is nil (eg. 20
    bytes of 0)
    """

    hash: 'HashOp' = betterproto.enum_field(6)
    """hash is the algorithm that must be used for each InnerOp"""


@dataclass(eq=False, repr=False)
class BatchProof(betterproto.Message):
    """BatchProof is a group of multiple proof types than can be compressed"""

    entries: List['BatchEntry'] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class BatchEntry(betterproto.Message):
    """Use BatchEntry not CommitmentProof, to avoid recursion"""

    exist: 'ExistenceProof' = betterproto.message_field(1, group='proof')
    nonexist: 'NonExistenceProof' = betterproto.message_field(2, group='proof')


@dataclass(eq=False, repr=False)
class CompressedBatchProof(betterproto.Message):
    entries: List['CompressedBatchEntry'] = betterproto.message_field(1)
    lookup_inners: List['InnerOp'] = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class CompressedBatchEntry(betterproto.Message):
    """Use BatchEntry not CommitmentProof, to avoid recursion"""

    exist: 'CompressedExistenceProof' = betterproto.message_field(1, group='proof')
    nonexist: 'CompressedNonExistenceProof' = betterproto.message_field(
        2, group='proof'
    )


@dataclass(eq=False, repr=False)
class CompressedExistenceProof(betterproto.Message):
    key: bytes = betterproto.bytes_field(1)
    value: bytes = betterproto.bytes_field(2)
    leaf: 'LeafOp' = betterproto.message_field(3)
    path: List[int] = betterproto.int32_field(4)
    """
    these are indexes into the lookup_inners table in CompressedBatchProof
    """


@dataclass(eq=False, repr=False)
class CompressedNonExistenceProof(betterproto.Message):
    key: bytes = betterproto.bytes_field(1)
    left: 'CompressedExistenceProof' = betterproto.message_field(2)
    right: 'CompressedExistenceProof' = betterproto.message_field(3)



from __future__ import annotations

from typing import Sequence

from evmos.proto.authz import (
    MsgGrant,
    MsgRevoke,
    RevokeMessages,
    StakeAuthTypes,
    create_msg_grant,
    create_msg_revoke,
    create_stake_authorization,
)
from evmos.proto.autogen.py.cosmos.bank.v1beta1 import MsgSend
from evmos.proto.autogen.py.cosmos.base.v1beta1 import Coin
from evmos.proto.autogen.py.cosmos.gov.v1beta1 import MsgVote, VoteOption
from evmos.proto.autogen.py.cosmos.tx.v1beta1 import AuthInfo, TxBody, TxRaw
from evmos.proto.autogen.py.ethermint.evm import v1 as ethermint
from evmos.proto.autogen.py.ethermint.types.v1 import ExtensionOptionsWeb3Tx
from evmos.proto.autogen.py.ibc.applications.transfer.v1 import MsgTransfer
from evmos.proto.autogen.py.ibc.core.client.v1 import Height
from evmos.proto.erc20 import (
    MsgConvertCoin,
    MsgConvertErc20,
    create_msg_convert_coin,
    create_msg_convert_erc20,
)
from evmos.proto.feesplit import (
    MsgCancelFeeSplit,
    MsgRegisterFeeSplit,
    MsgUpdateFeeSplit,
    create_msg_cancel_fee_split,
    create_msg_register_fee_split,
    create_msg_update_fee_split,
)
from evmos.proto.staking import (
    MsgBeginRedelegate,
    MsgDelegate,
    MsgUndelegate,
    MsgWithdrawDelegatorReward,
    MsgWithdrawValidatorCommission,
    create_msg_begin_redelegate,
    create_msg_delegate,
    create_msg_undelegate,
    create_msg_withdraw_delegator_reward,
    create_msg_withdraw_validator_commission,
)
from evmos.proto.transactions import (
    create_auth_info,
    create_body,
    create_body_with_multiple_messages,
    create_fee,
    create_sig_doc,
    create_signer_info,
    create_transaction,
    create_transaction_with_multiple_messages,
)
from evmos.proto.utils import MessageGenerated, create_any_message
from evmos.proto.validator import MsgEditValidator, create_msg_edit_validator

__all__ = [
    'create_msg_grant',
    'create_msg_revoke',
    'create_stake_authorization',
    'StakeAuthTypes',
    'RevokeMessages',
    'MsgConvertCoin',
    'MsgConvertErc20',
    'MsgGrant',
    'MsgRevoke',
    'MsgCancelFeeSplit',
    'MsgUpdateFeeSplit',
    'MsgRegisterFeeSplit',
    'MsgWithdrawDelegatorReward',
    'MsgWithdrawValidatorCommission',
    'MsgDelegate',
    'MsgBeginRedelegate',
    'MsgUndelegate',
    'MsgEditValidator',
    'TxRaw',
    'AuthInfo',
    'TxBody',
    'create_msg_convert_coin',
    'create_msg_convert_erc20',
    'create_msg_cancel_fee_split',
    'create_msg_register_fee_split',
    'create_msg_update_fee_split',
    'create_msg_delegate',
    'create_msg_begin_redelegate',
    'create_msg_undelegate',
    'create_msg_withdraw_delegator_reward',
    'create_msg_withdraw_validator_commission',
    'create_body_with_multiple_messages',
    'create_body',
    'create_fee',
    'create_signer_info',
    'create_auth_info',
    'create_sig_doc',
    'create_transaction_with_multiple_messages',
    'create_transaction',
    'MessageGenerated',
    'create_any_message',
    'create_msg_edit_validator',
    'create_msg_send',
    'create_ibc_msg_transfer',
    'bytes_to_msg_ethereum_tx',
    'bytes_to_legacy_tx',
    'bytes_to_access_list_tx',
    'bytes_to_dynamic_fee_tx',
    'bytes_to_tx_raw',
    'bytes_to_tx_body',
    'bytes_to_auth_info',
    'create_tx_raw',
    'create_msg_vote',
    'create_web3_extension',
]


# msgSend.ts


def create_msg_send(
    from_address: str,
    to_address: str,
    amount: str,
    denom: str,
) -> MessageGenerated[MsgSend]:
    """Create :class:`~evmos.proto.utils.MessageGenerated` for given args."""
    value = Coin(denom=denom, amount=amount)

    message = MsgSend(
        from_address=from_address,
        to_address=to_address,
        amount=[value],
    )
    return MessageGenerated(
        message=message,
        path='cosmos.bank.v1beta1.MsgSend',
    )


# ibcMsgTransfer.ts


def create_ibc_msg_transfer(
    # Channel
    source_port: str,
    source_channel: str,
    # Token
    amount: str,
    denom: str,
    # Addresses
    sender: str,
    receiver: str,
    # Timeout
    revision_number: int,
    revision_height: int,
    timeout_timestamp: str,
) -> MessageGenerated[MsgTransfer]:
    """Create message for IBC transfer."""
    token = Coin(denom=denom, amount=amount)

    timeout_height = Height(
        revision_number=revision_number,
        revision_height=revision_height,
    )

    ibc_message = MsgTransfer(
        source_port=source_port,
        source_channel=source_channel,
        token=token,
        sender=sender,
        receiver=receiver,
        timeout_height=timeout_height,
        timeout_timestamp=int(timeout_timestamp),
    )

    return MessageGenerated(
        message=ibc_message,
        path='ibc.applications.transfer.v1.MsgTransfer',
    )


# msgEthereumTx


def bytes_to_msg_ethereum_tx(binary_data: bytes) -> ethermint.MsgEthereumTx:
    """Deserialize :class:`evmos.proto.autogen.py.ethermint.evm.v1.MsgEthereumTx`."""
    return ethermint.MsgEthereumTx().parse(binary_data)


def bytes_to_legacy_tx(binary_data: bytes) -> ethermint.LegacyTx:
    """Deserialize :class:`evmos.proto.autogen.py.ethermint.evm.v1.LegacyTx`."""
    return ethermint.LegacyTx().parse(binary_data)


def bytes_to_access_list_tx(binary_data: bytes) -> ethermint.AccessListTx:
    """Deserialize :class:`evmos.proto.autogen.py.ethermint.evm.v1.AccessListTx`."""
    return ethermint.AccessListTx().parse(binary_data)


def bytes_to_dynamic_fee_tx(binary_data: bytes) -> ethermint.DynamicFeeTx:
    """Deserialize :class:`evmos.proto.autogen.py.ethermint.evm.v1.DynamicFeeTx`."""
    return ethermint.DynamicFeeTx().parse(binary_data)


# txRaw.ts


def bytes_to_tx_raw(binary_data: bytes) -> TxRaw:
    """Deserialize :class:`evmos.proto.autogen.py.cosmos.tx.v1beta1.TxRaw`."""
    return TxRaw().parse(binary_data)


def bytes_to_tx_body(binary_data: bytes) -> TxBody:
    """Deserialize :class:`evmos.proto.autogen.py.cosmos.tx.v1beta1.TxBody`."""
    return TxBody().parse(binary_data)


def bytes_to_auth_info(binary_data: bytes) -> AuthInfo:
    """Deserialize :class:`evmos.proto.autogen.py.cosmos.tx.v1beta1.AuthInfo`."""
    return AuthInfo().parse(binary_data)


def create_tx_raw(
    body_bytes: bytes,
    auth_info_bytes: bytes,
    signatures: Sequence[bytes],
) -> MessageGenerated[TxRaw]:
    """Create message with raw transaction."""
    message = TxRaw(
        body_bytes=body_bytes,
        auth_info_bytes=auth_info_bytes,
        signatures=list(signatures),
    )
    return MessageGenerated(
        message=message,
        path='cosmos.tx.v1beta1.TxRaw',
    )


# vote.ts


def create_msg_vote(
    proposal_id: int,
    option: int | VoteOption,
    sender: str,
) -> MessageGenerated[MsgVote]:
    """Create voting message."""
    vote_message = MsgVote(
        proposal_id=proposal_id,
        voter=sender,
        option=option,  # type: ignore[arg-type]
    )

    return MessageGenerated(
        message=vote_message,
        path='cosmos.gov.v1beta1.MsgVote',
    )


# web3Extension.ts


def create_web3_extension(
    chain_id: int,
    fee_payer: str,
    fee_payer_sig: bytes,
) -> MessageGenerated[ExtensionOptionsWeb3Tx]:
    """Create web3 extension options message."""
    message = ExtensionOptionsWeb3Tx(
        typed_data_chain_id=chain_id,
        fee_payer=fee_payer,
        fee_payer_sig=fee_payer_sig,
    )
    return MessageGenerated(
        message=message,
        path='ethermint.types.v1.ExtensionOptionsWeb3Tx',
    )



from __future__ import annotations

from dataclasses import dataclass
from typing import Any, Callable, Literal, Mapping, overload

import requests
from typing_extensions import Concatenate, ParamSpec

from evmos.eip712 import (
    EIPToSign,
    create_eip712,
    generate_fee,
    generate_message,
    generate_types,
)
from evmos.proto import MessageGenerated, create_transaction
from evmos.proto.transactions import TxGeneratedBase as TxGeneratedBase
from evmos.provider import generate_endpoint_account
from evmos.utils.doc import _inherit


@dataclass
class Fee:
    """Fee for message."""

    amount: str
    """Fee amount (stringified number, like '1000')."""
    denom: str
    """Denomination."""
    gas: str
    """Gas price."""


@dataclass
class Sender:
    """Message sender.

    ..versionadded:: 2.0.0
    """

    account_address: str
    """Account address (bech32, ``evmos1...``)."""
    sequence: int = 0
    """Account nonce - amount of previously sent transactions."""
    account_number: int = 0
    """Internal account number."""
    pubkey: str = ''
    """Account public key."""

    def update_from_chain(self, url: str = 'http://127.0.0.1:1317') -> None:
        """Set `sequence`, `account_number` and possibly `pubkey` from API response."""
        response = requests.get(
            f'{url}{generate_endpoint_account(self.account_address)}'
        )
        resp = response.json()

        self.sequence = int(resp['account']['base_account']['sequence'])
        self.account_number = int(resp['account']['base_account']['account_number'])
        if not self.pubkey:
            self.pubkey = resp['account']['base_account']['pub_key']['key']


@dataclass
class Chain:
    """Chain definition."""

    chain_id: int
    """Main chain ID."""
    cosmos_chain_id: str
    """Cosmos chain ID."""


@dataclass
class TxGenerated(TxGeneratedBase):
    """Transaction generated by this library (with EIP to sign)."""

    eip_to_sign: EIPToSign
    """EIP message to sign for EIP-712 transactions."""


_P = ParamSpec('_P')


def to_generated_base(
    func: Callable[Concatenate[str, _P], MessageGenerated[Any]]
) -> Callable[Concatenate[Chain, Sender, Fee, str, _P], TxGeneratedBase]:
    """Wrap function returning message with transaction base."""
    # Not using functools.wraps, because signature is altered
    @_inherit(func)
    def inner(
        chain: Chain,
        sender: Sender,
        fee: Fee,
        memo: str,
        *args: _P.args,
        **kwargs: _P.kwargs,
    ) -> TxGeneratedBase:
        msg_cosmos = func(sender.account_address, *args, **kwargs)

        tx = create_transaction(
            msg_cosmos,
            memo,
            fee.amount,
            fee.denom,
            int(fee.gas),
            'ethsecp256',
            sender.pubkey,
            sender.sequence,
            sender.account_number,
            chain.cosmos_chain_id,
        )

        return TxGeneratedBase(
            sign_direct=tx.sign_direct,
            legacy_amino=tx.legacy_amino,
        )

    return inner


@overload
def to_generated(
    types_def: dict[str, Any], proto: Literal[True]
) -> Callable[
    [Callable[Concatenate[str, _P], tuple[Mapping[str, Any], MessageGenerated[Any]]]],
    Callable[Concatenate[Chain, Sender, Fee, str, _P], TxGenerated],
]:
    ...


@overload
def to_generated(
    types_def: dict[str, Any], proto: Literal[False] = ...
) -> Callable[
    [Callable[_P, tuple[Mapping[str, Any], MessageGenerated[Any]]]],
    Callable[Concatenate[Chain, Sender, Fee, str, _P], TxGenerated],
]:
    ...


def to_generated(
    types_def: dict[str, Any], proto: bool = False
) -> Callable[
    [Callable[_P, tuple[Mapping[str, Any], MessageGenerated[Any]]]],
    Callable[Concatenate[Chain, Sender, Fee, str, _P], TxGenerated],
] | Callable[
    [Callable[Concatenate[str, _P], tuple[Mapping[str, Any], MessageGenerated[Any]]]],
    Callable[Concatenate[Chain, Sender, Fee, str, _P], TxGenerated],
]:
    """Wrap function returning message with transaction."""

    def _inner(
        chain: Chain,
        sender: Sender,
        fee: Fee,
        memo: str,
        msg: Mapping[str, Any],
        msg_cosmos: MessageGenerated[Any],
    ) -> TxGenerated:
        # EIP712
        fee_object = generate_fee(
            fee.amount,
            fee.denom,
            fee.gas,
            sender.account_address,
        )
        types = generate_types(types_def)

        messages = generate_message(
            str(sender.account_number),
            str(sender.sequence),
            chain.cosmos_chain_id,
            memo,
            fee_object,
            msg,
        )
        eip_to_sign = create_eip712(types, chain.chain_id, messages)

        # Cosmos

        tx = create_transaction(
            msg_cosmos,
            memo,
            fee.amount,
            fee.denom,
            int(fee.gas),
            'ethsecp256',
            sender.pubkey,
            sender.sequence,
            sender.account_number,
            chain.cosmos_chain_id,
        )

        return TxGenerated(
            sign_direct=tx.sign_direct,
            legacy_amino=tx.legacy_amino,
            eip_to_sign=eip_to_sign,
        )

    if proto:

        def decorator(
            func: Callable[
                Concatenate[str, _P], tuple[Mapping[str, Any], MessageGenerated[Any]]
            ]
        ) -> Callable[Concatenate[Chain, Sender, Fee, str, _P], TxGenerated]:
            @_inherit(func)
            def inner(
                chain: Chain,
                sender: Sender,
                fee: Fee,
                memo: str,
                *args: _P.args,
                **kwargs: _P.kwargs,
            ) -> TxGenerated:
                msg, msg_cosmos = func(sender.account_address, *args, **kwargs)
                return _inner(chain, sender, fee, memo, msg, msg_cosmos)

            return inner

        return decorator

    else:

        def decorator2(
            func: Callable[_P, tuple[Mapping[str, Any], MessageGenerated[Any]]]
        ) -> Callable[Concatenate[Chain, Sender, Fee, str, _P], TxGenerated]:
            @_inherit(func)
            def inner(
                chain: Chain,
                sender: Sender,
                fee: Fee,
                memo: str,
                *args: _P.args,
                **kwargs: _P.kwargs,
            ) -> TxGenerated:
                msg, msg_cosmos = func(*args, **kwargs)
                return _inner(chain, sender, fee, memo, msg, msg_cosmos)

            return inner

        return decorator2



from __future__ import annotations

from typing import Any, Mapping, Sequence

# from evmos.eip712 import MsgWithdrawDelegatorRewardInterface
from evmos.eip712 import (
    MSG_BEGIN_REDELEGATE_TYPES,
    MSG_DELEGATE_TYPES,
    MSG_UNDELEGATE_TYPES,
    MSG_WITHDRAW_DELEGATOR_REWARD_TYPES,
    MSG_WITHDRAW_VALIDATOR_COMMISSION_TYPES,
    create_eip712,
    create_msg_begin_redelegate,
    create_msg_delegate,
    create_msg_undelegate,
    create_msg_withdraw_delegator_reward,
    create_msg_withdraw_validator_commission,
    generate_fee,
    generate_message_with_multiple_transactions,
    generate_types,
)

# from evmos.proto import MsgWithdrawDelegatorRewardProtoInterface
from evmos.proto import (
    MessageGenerated,
    MsgBeginRedelegate,
    MsgDelegate,
    MsgUndelegate,
    MsgWithdrawDelegatorReward,
    MsgWithdrawValidatorCommission,
)
from evmos.proto import create_msg_begin_redelegate as proto_msg_begin_redelegate
from evmos.proto import create_msg_delegate as proto_msg_delegate
from evmos.proto import create_msg_undelegate as proto_msg_undelegate
from evmos.proto import (
    create_msg_withdraw_delegator_reward as proto_msg_withdraw_delegator_reward,
)
from evmos.proto import (
    create_msg_withdraw_validator_commission as proto_msg_withdraw_validator_commission,
)
from evmos.proto import create_transaction_with_multiple_messages
from evmos.transactions.common import Chain, Fee, Sender, TxGenerated, to_generated


@to_generated(MSG_DELEGATE_TYPES, proto=True)
def create_tx_msg_delegate(
    account_address: str,
    validator_address: str,
    amount: str,
    denom: str,
) -> tuple[Mapping[str, Any], MessageGenerated[MsgDelegate]]:
    """Create a transaction with delegation message."""
    # EIP712
    msg = create_msg_delegate(
        account_address,
        validator_address,
        amount,
        denom,
    )

    # Cosmos
    proto_message = proto_msg_delegate(
        account_address,
        validator_address,
        amount,
        denom,
    )
    return msg, proto_message


@to_generated(MSG_BEGIN_REDELEGATE_TYPES, proto=True)
def create_tx_msg_begin_redelegate(
    account_address: str,
    validator_src_address: str,
    validator_dst_address: str,
    amount: str,
    denom: str,
) -> tuple[Mapping[str, Any], MessageGenerated[MsgBeginRedelegate]]:
    """Create a transaction with redelegation beginning message."""
    # EIP712
    msg = create_msg_begin_redelegate(
        account_address,
        validator_src_address,
        validator_dst_address,
        amount,
        denom,
    )

    # Cosmos
    proto_message = proto_msg_begin_redelegate(
        account_address,
        validator_src_address,
        validator_dst_address,
        amount,
        denom,
    )
    return msg, proto_message


@to_generated(MSG_UNDELEGATE_TYPES, proto=True)
def create_tx_msg_undelegate(
    account_address: str,
    validator_address: str,
    amount: str,
    denom: str,
) -> tuple[Mapping[str, Any], MessageGenerated[MsgUndelegate]]:
    """Create a transaction with undelegation message."""
    # EIP712
    msg = create_msg_undelegate(
        account_address,
        validator_address,
        amount,
        denom,
    )

    # Cosmos
    proto_message = proto_msg_undelegate(
        account_address,
        validator_address,
        amount,
        denom,
    )
    return msg, proto_message


@to_generated(MSG_WITHDRAW_DELEGATOR_REWARD_TYPES, proto=True)
def create_tx_msg_withdraw_delegator_reward(
    account_address: str,
    validator_address: str,
) -> tuple[Mapping[str, Any], MessageGenerated[MsgWithdrawDelegatorReward]]:
    """Create a transaction with message for delegator reward withdrawal."""
    # EIP712
    msg = create_msg_withdraw_delegator_reward(
        account_address,
        validator_address,
    )

    # Cosmos
    proto_message = proto_msg_withdraw_delegator_reward(
        account_address,
        validator_address,
    )
    return msg, proto_message


# Multiple WithdrawRewards
def create_tx_msg_multiple_withdraw_delegator_reward(
    chain: Chain,
    sender: Sender,
    fee: Fee,
    memo: str,
    validator_addresses: Sequence[str],
) -> TxGenerated:
    """Create a transaction with message for delegator multiple rewards withdrawal."""
    # EIP712
    fee_object = generate_fee(
        fee.amount,
        fee.denom,
        fee.gas,
        sender.account_address,
    )
    types = generate_types(MSG_WITHDRAW_DELEGATOR_REWARD_TYPES)
    # EIP712
    # msgs: MsgWithdrawDelegatorRewardInterface[] = []
    msgs = []
    # Cosmos
    # proto_msgs: MsgWithdrawDelegatorRewardProtoInterface[] = []
    msgs = [
        create_msg_withdraw_delegator_reward(sender.account_address, validator)
        for validator in validator_addresses
    ]
    proto_msgs = [
        proto_msg_withdraw_delegator_reward(sender.account_address, validator)
        for validator in validator_addresses
    ]

    messages = generate_message_with_multiple_transactions(
        str(sender.account_number),
        str(sender.sequence),
        chain.cosmos_chain_id,
        memo,
        fee_object,
        msgs,
    )
    eip_to_sign = create_eip712(types, chain.chain_id, messages)

    # Cosmos
    tx = create_transaction_with_multiple_messages(
        proto_msgs,
        memo,
        fee.amount,
        fee.denom,
        int(fee.gas),
        'ethsecp256',
        sender.pubkey,
        sender.sequence,
        sender.account_number,
        chain.cosmos_chain_id,
    )

    return TxGenerated(
        sign_direct=tx.sign_direct,
        legacy_amino=tx.legacy_amino,
        eip_to_sign=eip_to_sign,
    )


@to_generated(MSG_WITHDRAW_VALIDATOR_COMMISSION_TYPES)
def create_tx_msg_withdraw_validator_commission(
    validator_address: str,
) -> tuple[Mapping[str, Any], MessageGenerated[MsgWithdrawValidatorCommission]]:
    """Create a transaction with message for validator commission withdrawal."""
    # EIP712
    msg = create_msg_withdraw_validator_commission(validator_address)

    # Cosmos
    proto_message = proto_msg_withdraw_validator_commission(validator_address)
    return msg, proto_message



from __future__ import annotations

from evmos.constants import NOT_MODIFY
from evmos.proto.autogen.py.cosmos.staking.v1beta1 import Description, MsgEditValidator
from evmos.proto.utils import MessageGenerated


def create_msg_edit_validator(
    *,
    moniker: str | None,
    identity: str | None,
    website: str | None,
    security_contact: str | None,
    details: str | None,
    validator_address: str,
    commission_rate: str | None,
    min_self_delegation: str | None,
) -> MessageGenerated[MsgEditValidator]:
    """Create message for validator editing."""
    message = MsgEditValidator(
        description=Description(
            moniker=moniker or NOT_MODIFY,
            identity=identity or NOT_MODIFY,
            website=website or NOT_MODIFY,
            security_contact=security_contact or NOT_MODIFY,
            details=details or NOT_MODIFY,
        ),
        validator_address=validator_address,
        commission_rate=str(commission_rate or '<nil>'),
        min_self_delegation=str(min_self_delegation or '<nil>'),
    )

    return MessageGenerated(
        message=message,
        path='cosmos.staking.v1beta1.MsgEditValidator',
    )



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: evmos/incentives/v1/genesis.proto, evmos/incentives/v1/incentives.proto, evmos/incentives/v1/query.proto
# plugin: python-betterproto
# This file has been @generated
from dataclasses import dataclass
from datetime import datetime
from typing import (
    TYPE_CHECKING,
    Dict,
    List,
    Optional,
)

import betterproto
import grpclib
from betterproto.grpc.grpclib_server import ServiceBase

from ....cosmos.base import v1beta1 as ___cosmos_base_v1_beta1__
from ....cosmos.base.query import v1beta1 as ___cosmos_base_query_v1_beta1__


if TYPE_CHECKING:
    import grpclib.server
    from betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class Incentive(betterproto.Message):
    """
    Incentive defines an instance that organizes distribution conditions for a
    given smart contract
    """

    contract: str = betterproto.string_field(1)
    """contract address"""

    allocations: List['___cosmos_base_v1_beta1__.DecCoin'] = betterproto.message_field(
        2
    )
    """denoms and percentage of rewards to be allocated"""

    epochs: int = betterproto.uint32_field(3)
    """number of remaining epochs"""

    start_time: datetime = betterproto.message_field(4)
    """distribution start time"""

    total_gas: int = betterproto.uint64_field(5)
    """
    cumulative gas spent by all gasmeters of the incentive during the epoch
    """


@dataclass(eq=False, repr=False)
class GasMeter(betterproto.Message):
    """
    GasMeter tracks the cumulative gas spent per participant in one epoch
    """

    contract: str = betterproto.string_field(1)
    """hex address of the incentivized contract"""

    participant: str = betterproto.string_field(2)
    """participant address that interacts with the incentive"""

    cumulative_gas: int = betterproto.uint64_field(3)
    """cumulative gas spent during the epoch"""


@dataclass(eq=False, repr=False)
class RegisterIncentiveProposal(betterproto.Message):
    """
    RegisterIncentiveProposal is a gov Content type to register an incentive
    """

    title: str = betterproto.string_field(1)
    """title of the proposal"""

    description: str = betterproto.string_field(2)
    """proposal description"""

    contract: str = betterproto.string_field(3)
    """contract address"""

    allocations: List['___cosmos_base_v1_beta1__.DecCoin'] = betterproto.message_field(
        4
    )
    """denoms and percentage of rewards to be allocated"""

    epochs: int = betterproto.uint32_field(5)
    """number of remaining epochs"""


@dataclass(eq=False, repr=False)
class CancelIncentiveProposal(betterproto.Message):
    """CancelIncentiveProposal is a gov Content type to cancel an incentive"""

    title: str = betterproto.string_field(1)
    """title of the proposal"""

    description: str = betterproto.string_field(2)
    """proposal description"""

    contract: str = betterproto.string_field(3)
    """contract address"""


@dataclass(eq=False, repr=False)
class GenesisState(betterproto.Message):
    """GenesisState defines the module's genesis state."""

    params: 'Params' = betterproto.message_field(1)
    """module parameters"""

    incentives: List['Incentive'] = betterproto.message_field(2)
    """active incentives"""

    gas_meters: List['GasMeter'] = betterproto.message_field(3)
    """active Gasmeters"""


@dataclass(eq=False, repr=False)
class Params(betterproto.Message):
    """Params defines the incentives module params"""

    enable_incentives: bool = betterproto.bool_field(1)
    """parameter to enable incentives"""

    allocation_limit: str = betterproto.string_field(2)
    """maximum percentage an incentive can allocate per denomination"""

    incentives_epoch_identifier: str = betterproto.string_field(3)
    """identifier for the epochs module hooks"""

    reward_scaler: str = betterproto.string_field(4)
    """scaling factor for capping rewards"""


@dataclass(eq=False, repr=False)
class QueryIncentivesRequest(betterproto.Message):
    """
    QueryIncentivesRequest is the request type for the Query/Incentives RPC
    method.
    """

    pagination: '___cosmos_base_query_v1_beta1__.PageRequest' = (
        betterproto.message_field(1)
    )
    """pagination defines an optional pagination for the request."""


@dataclass(eq=False, repr=False)
class QueryIncentivesResponse(betterproto.Message):
    """
    QueryIncentivesResponse is the response type for the Query/Incentives RPC
    method.
    """

    incentives: List['Incentive'] = betterproto.message_field(1)
    pagination: '___cosmos_base_query_v1_beta1__.PageResponse' = (
        betterproto.message_field(2)
    )
    """pagination defines the pagination in the response."""


@dataclass(eq=False, repr=False)
class QueryIncentiveRequest(betterproto.Message):
    """
    QueryIncentiveRequest is the request type for the Query/Incentive RPC method.
    """

    contract: str = betterproto.string_field(1)
    """contract identifier is the hex contract address of a contract"""


@dataclass(eq=False, repr=False)
class QueryIncentiveResponse(betterproto.Message):
    """
    QueryIncentiveResponse is the response type for the Query/Incentive RPC
    method.
    """

    incentive: 'Incentive' = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class QueryGasMetersRequest(betterproto.Message):
    """
    QueryGasMetersRequest is the request type for the Query/Incentives RPC
    method.
    """

    contract: str = betterproto.string_field(1)
    """
    contract is the hex contract address of a incentivized smart contract
    """

    pagination: '___cosmos_base_query_v1_beta1__.PageRequest' = (
        betterproto.message_field(2)
    )
    """pagination defines an optional pagination for the request."""


@dataclass(eq=False, repr=False)
class QueryGasMetersResponse(betterproto.Message):
    """
    QueryGasMetersResponse is the response type for the Query/Incentives RPC
    method.
    """

    gas_meters: List['GasMeter'] = betterproto.message_field(1)
    pagination: '___cosmos_base_query_v1_beta1__.PageResponse' = (
        betterproto.message_field(2)
    )
    """pagination defines the pagination in the response."""


@dataclass(eq=False, repr=False)
class QueryGasMeterRequest(betterproto.Message):
    """
    QueryGasMeterRequest is the request type for the Query/Incentive RPC method.
    """

    contract: str = betterproto.string_field(1)
    """contract identifier is the hex contract address of a contract"""

    participant: str = betterproto.string_field(2)
    """participant identifier is the hex address of a user"""


@dataclass(eq=False, repr=False)
class QueryGasMeterResponse(betterproto.Message):
    """
    QueryGasMeterResponse is the response type for the Query/Incentive RPC
    method.
    """

    gas_meter: int = betterproto.uint64_field(1)


@dataclass(eq=False, repr=False)
class QueryAllocationMetersRequest(betterproto.Message):
    """
    QueryAllocationMetersRequest is the request type for the
    Query/AllocationMeters RPC method.
    """

    pagination: '___cosmos_base_query_v1_beta1__.PageRequest' = (
        betterproto.message_field(1)
    )
    """pagination defines an optional pagination for the request."""


@dataclass(eq=False, repr=False)
class QueryAllocationMetersResponse(betterproto.Message):
    """
    QueryAllocationMetersResponse is the response type for the
    Query/AllocationMeters RPC method.
    """

    allocation_meters: List[
        '___cosmos_base_v1_beta1__.DecCoin'
    ] = betterproto.message_field(1)
    pagination: '___cosmos_base_query_v1_beta1__.PageResponse' = (
        betterproto.message_field(2)
    )
    """pagination defines the pagination in the response."""


@dataclass(eq=False, repr=False)
class QueryAllocationMeterRequest(betterproto.Message):
    """
    QueryAllocationMeterRequest is the request type for the Query/AllocationMeter
    RPC method.
    """

    denom: str = betterproto.string_field(1)
    """denom is the coin denom to query an allocation meter for."""


@dataclass(eq=False, repr=False)
class QueryAllocationMeterResponse(betterproto.Message):
    """
    QueryAllocationMeterResponse is the response type for the
    Query/AllocationMeter RPC method.
    """

    allocation_meter: '___cosmos_base_v1_beta1__.DecCoin' = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class QueryParamsRequest(betterproto.Message):
    """
    QueryParamsRequest is the request type for the Query/Params RPC method.
    """

    pass


@dataclass(eq=False, repr=False)
class QueryParamsResponse(betterproto.Message):
    """
    QueryParamsResponse is the response type for the Query/Params RPC
    method.
    """

    params: 'Params' = betterproto.message_field(1)


class QueryStub(betterproto.ServiceStub):
    async def incentives(
        self,
        query_incentives_request: 'QueryIncentivesRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryIncentivesResponse':
        return await self._unary_unary(
            '/evmos.incentives.v1.Query/Incentives',
            query_incentives_request,
            QueryIncentivesResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def incentive(
        self,
        query_incentive_request: 'QueryIncentiveRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryIncentiveResponse':
        return await self._unary_unary(
            '/evmos.incentives.v1.Query/Incentive',
            query_incentive_request,
            QueryIncentiveResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def gas_meters(
        self,
        query_gas_meters_request: 'QueryGasMetersRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryGasMetersResponse':
        return await self._unary_unary(
            '/evmos.incentives.v1.Query/GasMeters',
            query_gas_meters_request,
            QueryGasMetersResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def gas_meter(
        self,
        query_gas_meter_request: 'QueryGasMeterRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryGasMeterResponse':
        return await self._unary_unary(
            '/evmos.incentives.v1.Query/GasMeter',
            query_gas_meter_request,
            QueryGasMeterResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def allocation_meters(
        self,
        query_allocation_meters_request: 'QueryAllocationMetersRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryAllocationMetersResponse':
        return await self._unary_unary(
            '/evmos.incentives.v1.Query/AllocationMeters',
            query_allocation_meters_request,
            QueryAllocationMetersResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def allocation_meter(
        self,
        query_allocation_meter_request: 'QueryAllocationMeterRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryAllocationMeterResponse':
        return await self._unary_unary(
            '/evmos.incentives.v1.Query/AllocationMeter',
            query_allocation_meter_request,
            QueryAllocationMeterResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def params(
        self,
        query_params_request: 'QueryParamsRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryParamsResponse':
        return await self._unary_unary(
            '/evmos.incentives.v1.Query/Params',
            query_params_request,
            QueryParamsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class QueryBase(ServiceBase):
    async def incentives(
        self, query_incentives_request: 'QueryIncentivesRequest'
    ) -> 'QueryIncentivesResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def incentive(
        self, query_incentive_request: 'QueryIncentiveRequest'
    ) -> 'QueryIncentiveResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def gas_meters(
        self, query_gas_meters_request: 'QueryGasMetersRequest'
    ) -> 'QueryGasMetersResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def gas_meter(
        self, query_gas_meter_request: 'QueryGasMeterRequest'
    ) -> 'QueryGasMeterResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def allocation_meters(
        self, query_allocation_meters_request: 'QueryAllocationMetersRequest'
    ) -> 'QueryAllocationMetersResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def allocation_meter(
        self, query_allocation_meter_request: 'QueryAllocationMeterRequest'
    ) -> 'QueryAllocationMeterResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def params(
        self, query_params_request: 'QueryParamsRequest'
    ) -> 'QueryParamsResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_incentives(
        self,
        stream: 'grpclib.server.Stream[QueryIncentivesRequest, QueryIncentivesResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.incentives(request)
        await stream.send_message(response)

    async def __rpc_incentive(
        self,
        stream: 'grpclib.server.Stream[QueryIncentiveRequest, QueryIncentiveResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.incentive(request)
        await stream.send_message(response)

    async def __rpc_gas_meters(
        self,
        stream: 'grpclib.server.Stream[QueryGasMetersRequest, QueryGasMetersResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.gas_meters(request)
        await stream.send_message(response)

    async def __rpc_gas_meter(
        self,
        stream: 'grpclib.server.Stream[QueryGasMeterRequest, QueryGasMeterResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.gas_meter(request)
        await stream.send_message(response)

    async def __rpc_allocation_meters(
        self,
        stream: 'grpclib.server.Stream[QueryAllocationMetersRequest, QueryAllocationMetersResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.allocation_meters(request)
        await stream.send_message(response)

    async def __rpc_allocation_meter(
        self,
        stream: 'grpclib.server.Stream[QueryAllocationMeterRequest, QueryAllocationMeterResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.allocation_meter(request)
        await stream.send_message(response)

    async def __rpc_params(
        self, stream: 'grpclib.server.Stream[QueryParamsRequest, QueryParamsResponse]'
    ) -> None:
        request = await stream.recv_message()
        response = await self.params(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            '/evmos.incentives.v1.Query/Incentives': grpclib.const.Handler(
                self.__rpc_incentives,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryIncentivesRequest,
                QueryIncentivesResponse,
            ),
            '/evmos.incentives.v1.Query/Incentive': grpclib.const.Handler(
                self.__rpc_incentive,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryIncentiveRequest,
                QueryIncentiveResponse,
            ),
            '/evmos.incentives.v1.Query/GasMeters': grpclib.const.Handler(
                self.__rpc_gas_meters,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryGasMetersRequest,
                QueryGasMetersResponse,
            ),
            '/evmos.incentives.v1.Query/GasMeter': grpclib.const.Handler(
                self.__rpc_gas_meter,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryGasMeterRequest,
                QueryGasMeterResponse,
            ),
            '/evmos.incentives.v1.Query/AllocationMeters': grpclib.const.Handler(
                self.__rpc_allocation_meters,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryAllocationMetersRequest,
                QueryAllocationMetersResponse,
            ),
            '/evmos.incentives.v1.Query/AllocationMeter': grpclib.const.Handler(
                self.__rpc_allocation_meter,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryAllocationMeterRequest,
                QueryAllocationMeterResponse,
            ),
            '/evmos.incentives.v1.Query/Params': grpclib.const.Handler(
                self.__rpc_params,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryParamsRequest,
                QueryParamsResponse,
            ),
        }
